/**
 * Helper class for grid-related preferences (dgrids in general, not just CPB DataGrids).
 */
define("sas-cpb/prefs/GridPrefs", [ '../cpb', './LocalPrefs', 'dojo/dom-style', 'dgrid/util/misc' ],
   function(cpb, LocalPrefs, domStyle, miscUtil) {

   'use strict';

   var GridPrefs = {};

   var _isColumnVisibleInEditModeOnly = function(fieldName) {
      return fieldName==='checkboxes' || fieldName==='actionMenuColumn';
   };

   var _isFixedColumn = function(col) {
      return col.resizable===false;//fieldName==='checkboxes' || fieldName==='actionMenuColumn';
   };

   var _getColumnOrder = function(grid, forceResizeCalc) {

      var colInfos = [];
      var orderedCols = grid.get('subRows')[0];

      for (var i=0; i<orderedCols.length; i++) {

         var col = orderedCols[i];
         if (_isColumnVisibleInEditModeOnly(col.field)) {
            continue; // We don't record info for edit-mode specific columns.
         }
         var colInfo = { id: col.field };

         if (!_isFixedColumn(col)) {

            if (col.hidden) {
               colInfo.hidden = true;
            }
            else if (!col.width || col.width==='auto') {
               // Last column is always set to auto
               if (forceResizeCalc && i<orderedCols.length-1) {
                  var newWidth = domStyle.get(col.headerNode, 'width');
                  console.log('Setting column with bad width (' + col.width + ') to ' + newWidth + ': ' + col.field);
                  col.width = newWidth;
               }
               else {
                  console.log('Setting column with bad width (' + col.width + ') to auto: ' + col.field);
                  col.width = 'auto';
               }
            }

            if (!col.hidden) { // is this check necessary?
               colInfo.w = col.width;
            }

         }

         colInfos.push(colInfo);
      }
      return colInfos;
   };

   var _getSortInfo = function(grid) {
      var sortInfo = [];
      grid.get('sort').forEach(function (colInfo) {
         sortInfo.push({ attribute: colInfo.attribute, descending: colInfo.descending });
      });
      return sortInfo;
   };

   var _getPrefsPrefixImpl = function() {
      var prefix = cpb.preferencesPrefix;
      if (!prefix) {
         cpb.consoleLog('No cpb.preferencesPrefix found!  Grid preferences may not be accurate.', 'error');
         prefix = '';
      }
      return prefix;
   };

   var _getGridPrefsKey = function(gridName) {
      var prefix = _getPrefsPrefixImpl();
      return prefix + '-' + gridName;
   };

   /**
    * Removes any columns from sortInfo that are not in availableColumns.  This prevents us from asking the server to
    * sort on a (potentially) invalid column.
    *
    * @param sortInfo Locally cached grid sort info (e.g. from local or session storage).
    * @param availableColumns The search grid's entire column set, as sent down by the server.
    * @returns {Array} The subset of sortInfo that was found in availableColumns.
    * @private
    */
   var _removeUnavailableColumns = function(sortInfo, availableColumns) {
      var cleansedSortInfo = [];
      for (var i=0; i<sortInfo.length; i++) {
         var field = sortInfo[i].attribute;
         var found = false;
         for (var j=0; j<availableColumns.length; j++) {
            if (availableColumns[j].field === field) {
               cleansedSortInfo.push(sortInfo[i]);
               found = true;
               break;
            }
         }
         if (!found) {
            // This should indicate that available columns (search grid config) changed since we last saved our prefs.
            cpb.consoleLog('Removing cached sort column "' + field + '" as it is not in available column set', 'warn');
         }
      }
      return cleansedSortInfo;
   };

   /**
    * Loads a grid's column sizes, visibility, and sort information from local storage.  This information is merged
    * into the specified "args" parameter.  If a user does not have saved column widths for this grid,
    * server-specified defaults are used.
    */
   GridPrefs.loadGridPrefs = function(args, expectedVersion) {

      var gridName = args.cpb_name || args.id;

      var prefs = LocalPrefs.getItem(_getGridPrefsKey(gridName));
      if (prefs) {
         if (expectedVersion && prefs.v!==expectedVersion) {
            // Future releases might be able to massage older data versions into newer formats
            cpb.consoleLog('Grid preferences are from an old version and cannot be loaded: ' + gridName);
         }
         else {
            _loadGridSortPrefs(gridName, prefs, args);
         }
      }

      // We always load column preferences, as this also merges server-specified default widths if user-cached
      // column widths are not found.
      _loadGridColumnPrefs(gridName, prefs, args.columns);
   };

   /**
    * Saves a grid's column sizes, visibility, and sort information.
    */
   GridPrefs.saveGridPrefs = function(grid, forceResizeCalc) {
      // Wait for column div to actually resize; event is fired early (!)
      setTimeout(function() {
         var data = {
            v: 1,
            columns: _getColumnOrder(grid, forceResizeCalc),
            sort: _getSortInfo(grid)
         };
         var keySuffix = grid.cpb_name || grid.id;
         LocalPrefs.setItem(_getGridPrefsKey(keySuffix), data);
      }, 1);
   };

   var _getColumnIndex = function(orderedColInfos, colName) {
      for (var i=0; i<orderedColInfos.length; i++) {
         if (orderedColInfos[i].id === colName) {
            return i;
         }
      }
      return -1;
   };

   /**
    * Loads a grid's sort preferences from gridPrefs and modifies gridArgs to reflect them.
    */
   var _loadGridSortPrefs = function(gridName, gridPrefs, gridArgs) {
      var sortInfo = gridPrefs ? gridPrefs.sort : null;
      if (sortInfo) {
         var orig = gridArgs.sort ? gridArgs.sort.slice(0) : undefined;
         gridArgs.sort = _removeUnavailableColumns(sortInfo, gridArgs.columns);
         if (gridArgs.sort.length !== sortInfo.length) {
            // A no-longer valid column was discovered and removed - bail on sorting
            cpb.consoleLog('Grid ' + gridName + ' sorts on columns no longer available from server.  Ignoring sort cache.');
            // dgrid seems unhappy with this member set to undefined
            if (orig) {
               gridArgs.sort = orig;
            }
            else {
               delete gridArgs.sort;
            }
         }
      }
   };

   /**
    * Registers a listener that saves this grid's column size and sort info whenever it is changed.
    */
   GridPrefs.registerPrefsListener = function(grid) {

      var saveGridPrefs = function(e) {
         // dgrid-columnresize doesn't need a recalculation as only affected columns have their widths modified
         var forceResizeCalc = e &&
             (e.type==='dgrid-sort' || e.type==='dgrid-columnstatechange' || e.type==='dgrid-columnreorder');
         console.log('Forcing column recalculation: ' + forceResizeCalc);
         GridPrefs.saveGridPrefs(grid, forceResizeCalc);
      };

      grid.on('dgrid-sort', saveGridPrefs);
      grid.on('dgrid-columnresize', saveGridPrefs);
      grid.on('dgrid-columnstatechange', saveGridPrefs); // Column hidden or shown
      grid.on('dgrid-columnreorder', saveGridPrefs);
   };

   /**
    * Loads grid column preferences and mixes them into a grid's column model.
    */
   var _loadGridColumnPrefs = function(gridName, gridPrefs, columnModel) {

      var orderedColumnPrefs = gridPrefs ? gridPrefs.columns : null;
      var i, colModel, colWidth;

      if (orderedColumnPrefs) {

         var ordered = _orderColumns(gridName, columnModel, orderedColumnPrefs);
         if (!ordered) {
            return false; // Server sent different column set than was saved - don't sort and don't size
         }

         for (i=0; i<columnModel.length; i++) {
            colModel = columnModel[i];
            var index = _getColumnIndex(orderedColumnPrefs, colModel.field);
            if (index>-1) {
               var savedColInfo = orderedColumnPrefs[index];
               if (savedColInfo.w) { // Not set if hidden
                  colWidth = savedColInfo.w==='auto' ? 'auto' :
                        (/(?:px|%)$/.test(savedColInfo.w) ? savedColInfo.w : (savedColInfo.w + 'px'));
                  colModel.startW = colWidth; // For testing purposes only; startW not used in app or dgrid
                  miscUtil.addCssRule("#" + miscUtil.escapeCssIdentifier(gridName) + " .dgrid-column-" + i,
                              "width: " + colWidth + ";");
               }
               if (savedColInfo.hidden) {
                  colModel.hidden = true;
               }
            }
         }
      }

      else {

         // No saved column preferences - use server-specified default widths if available
         for (i=0; i<columnModel.length; i++) {
            colModel = columnModel[i];
            if (colModel.defaultWidth) {
               colWidth = colModel.defaultWidth==='auto' ? 'auto' : (colModel.defaultWidth + '%');
               colModel.startW = colWidth; // For testing purposes only; startW not used in app or dgrid
               miscUtil.addCssRule("#" + miscUtil.escapeCssIdentifier(gridName) + " .dgrid-column-" + i,
                           "width: " + colWidth + ";");
            }
         }
      }

      return orderedColumnPrefs !== null;
   };

   var _orderColumns = function(gridName, columnModel, colOrder) {

      // Since we don't record column info for checkbox columns or action menu columns,
      // we must skip over those columns (if they exist) in the column model from the server.
      // They're configured to always be left-aligned, fixed width and unsortable anyway.
      var colSkipCount = 0;
      while (colSkipCount<columnModel.length && _isColumnVisibleInEditModeOnly(columnModel[colSkipCount].field)) {
         colSkipCount++;
      }

      // First, ensure we have the same set of columns specified
      if ((columnModel.length-colSkipCount) !== colOrder.length) {
         cpb.consoleLog('Column model for grid ' + gridName + ' (length===' + columnModel.length + ', but ignoring ' +
               colSkipCount + ' leading columns) is a different size than cache (' + colOrder.length +
               ').  Previous column order will be discarded.');
         return false;
      }

      var colOrderCopy = colOrder.slice(0);
      for (var i=colSkipCount; i<columnModel.length; i++) {
         var colName = columnModel[i].field;
         var index = _getColumnIndex(colOrderCopy, colName);
         if (index===-1) {
            cpb.consoleLog('Grid ' + gridName + ' has different columns than those specified in cache.  Ignoring order cache.');
            return false;
         }
         colOrderCopy.splice(index, 1);
      }

      // Now, order them
      var leadingCols = columnModel.slice(0, colSkipCount);
      var sortableCols = columnModel.slice(colSkipCount);
      sortableCols.sort(function(a, b) {
         return _getColumnIndex(colOrder,  a.field) - _getColumnIndex(colOrder, b.field);
      });
      columnModel.splice(0,columnModel.length);
      columnModel.push.apply(columnModel, leadingCols.concat(sortableCols));

      return true;
   };

   GridPrefs.clearPageGridPreferences = function() {
      var prefix = _getPrefsPrefixImpl();
      if (prefix && prefix.length>0) {
         var removeCount = LocalPrefs.clearItemsWithPrefix(prefix);
         cpb.consoleLog('Removed ' + removeCount + ' preferences items for this page from localStorage');
      }
      else {
         cpb.consoleLog('Not clearing page preferences since no preferences prefix was found', 'error');
      }
   };

   GridPrefs.clearAllGridPreferences = function() {
      var removeCount = LocalPrefs.clear();
      cpb.consoleLog('Removed ' + removeCount + ' preferences items for this application from localStorage');
   };

   return GridPrefs;

});
