
/***********************************************************************************/
/* Copyright (c) 2017, SAS Institute Inc., Cary, NC, USA, All Rights Reserved      */
/*                                                                                 */
/* NAME:          ecmrr_efile_fincen_ctrx_driver                                   */
/* PRODUCT:       Enterprise Case Management                                       */
/* PURPOSE:       Driver program for generating FINCEN CTRX XML e-files            */
/*                after the data is extracted.                                     */
/* PARAMETERS:                                                                     */
/*                form_config_rk: form configuration key                           */
/* NOTE:                                                                           */
/***********************************************************************************/

%macro ecmrr_efile_fincen_ctrx_driver(form_config_rk=212);

%put %sysfunc(sasmsg(&smdds,GEN_MAC_START_NOTE,NOQUOTE,&sysmacroname)) ;

%if %symexist(efile_rk)=0 %then
%do; 
    %let successYN=N;	
    %put ERROR: efile_rk is required. ;
    %goto mExit;
%end;

%if %nrstr(&efile_rk) = %nrstr() %then 
%do; 
    %let successYN=N;
	 %put ERROR: efile_rk is required. ;
    %goto mExit;
%end;

%let successYN=Y; 

%let efile_file_prefix=;
%let efile_form_type_cd=;

    /* retrieve efile config info */ 
     proc sql noprint;
         select trim(efile_file_prefix),
                upcase(trim(form_type_cd))
           into :efile_file_prefix,
                :efile_form_type_cd
         from ecm_db.form_config
         where form_config_rk=&form_config_rk;
      
     quit;
 

%if &sqlrc > 4 %then 
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;

      /* Determine the minimum and maximum transaction date for reports in efile */
      proc sql noprint;
            select max(udf_value_str),
                   min(udf_value_str)
            into :X_MAX_TRAN_DT,
                 :X_MIN_TRAN_DT
            from rr_udf_date_value
               where  udf_table_nm = 'RR'
                  and udf_nm = 'X_TRANSACTION_DT'
            ;
      quit;

%if &sqlrc > 4 %then
%do;
     %let successYN=N;
     %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto mExit;
%end;

%if &ecm_debug=Y %then
%do;
    %put Maximum efile transmission date: &X_MAX_TRAN_DT;
    %put Minimum efile transmission date: &X_MIN_TRAN_DT;
%end;

     /************************************************************/ 
     /* Define efile_output_info table for stored process output */
     /************************************************************/ 

       data efile_output_info;
            set ecm_db.efile_live ;
            where efile_rk=&efile_rk;                  
        
         length x_output_zip_nm $ 64
                x_sdtm_efile_nm $ 100
                ;
         output_nm=strip(efile_id)||'.xml';
         output_create_dttm=datetime();         
         output_path_nm="&efile_file_prefix";       
         output_rr_cnt=%ecm_nobs(rr_subset);          
         coverage_start_dt=input("&X_MIN_TRAN_DT",yymmdd8.)*3600*24;
         coverage_end_dt=input("&X_MAX_TRAN_DT",yymmdd8.)*3600*24; 
         output_zip_nm='';         
         call symput('output_path_nm',strip(output_path_nm));
         call symput('output_nm',strip(output_nm));
      run;  
      
      %if &syserr > 4 %then 
      %do;
          %let successYN=N;
          %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
          %goto mExit;
      %end;
    

    
	/***********************************************************/
	/* use PROC XSL to transform the xml in udf fields into    */
	/* FinCEN required format xml.                             */
	/***********************************************************/

	%let efile_subpath=%trim(&ecm_efile_path)&slash.%trim(&output_path_nm);
    %let form_path=&ecm_config_path&slash.Source&slash.form_template;

	filename xslt "&form_path&slash%lowcase(&efile_form_type_cd)_efile.xsl";
  
    filename rawxml "%sysfunc(pathname(work))&slash.ctrxRaw.xml";
    filename efilenm "&efile_subpath&slash.&output_nm";
    filename outxml "%sysfunc(pathname(work))&slash.ctrxTemp.xml";
	
	proc xsl
	   in=rawxml
	   xsl=xslt
	   out=outxml
	   ;
	run;

    %if &syserr > 4 %then 
    %do;
        %let successYN=N;
        %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
        %goto mExit;
    %end;
           
   /******************************************/
   /* add unique sequence numbers to the xml */
   /******************************************/

   %let line_cnt=0;
   %ecmrr_efile_xml_seqnum(exclude_dsn=z_rr_efiled, 
                           exclude_var=rr_rk,
                           infname=outxml, 
                           outfname=efilenm, 
                           lineCntVar=line_cnt);

 
   %if &successYN=N %then %goto mExit;
             
        
   /* obtain file size and creation datetime */
   %let output_create_dttm=%sysfunc(datetime());
   data _null_;
        fid=fopen('efilenm');
        call symput('output_file_size',strip(finfo(fid,'File Size (bytes)')));
        rc=fclose (fid);
   run; 

   %if &syserr > 4 %then 
   %do;
       %let successYN=N;
       %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
       %goto mExit;
   %end;

   %if &ecm_debug ne Y %then
   %do;
       filename efilenm clear;
   %end;
   /* update efile_output_info with the file size and actual creation datetime */
   proc sql;
        update efile_output_info
        set output_file_size=&output_file_size,
            output_create_dttm=&output_create_dttm,
            output_line_cnt=&line_cnt
        ;
   quit;

   %if &syserr > 4 %then 
   %do;
       %let successYN=N;
       %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
       %goto mExit;
   %end;     
 


%if %upcase(&&&ecm_sdtm_YN_&efile_form_type_cd)=Y %then 
%do;
      /* create same efile for sdtm */
     %ecmrr_efile_fincen_sdtm(form_type_cd=&efile_form_type_cd,
                              efile_rk=&efile_rk,
                              efile_info_dsn=efile_output_info
                              );
     
%end;

%if &successYN=N %then %goto mExit;


%if %upcase(&ecm_run_soap=Y) %then
%do; 
     /* update report data with seq number and x_filed_dt */     
     %ecmrr_efile_update_rr (form_config_rk=,
                             rr_indsn=z_rr_efiled,
                             efile_dt_var=,
                             efile_dt_value=
                            );
%end;

%if &successYN=N %then %goto mExit;
  
                      
%mexit:

%put %sysfunc(sasmsg(&smdds,GEN_MAC_END_NOTE,NOQUOTE,&sysmacroname)) ;

%mend;
