﻿
/***********************************************************************************/
/* Copyright (c) 2018, SAS Institute Inc., Cary, NC, USA, All Rights Reserved      */
/*                                                                                 */
/* NAME:          ecmrr_efile_fincen_sarx_sum                                      */
/* PRODUCT:       Enterprise Case Management                                       */
/* PURPOSE:       Convert udf name and value pairs into xml                        */
/* PARAMETERS:                                                                     */
/*                rr_udf_dsn: name of dataset with selected report data            */
/*                outfref: file reference of the output xml                        */
/*                x_filed_dt: date used as filing date                             */
/***********************************************************************************/

%macro ecmrr_efile_fincen_sarx_sum(rr_udf_dsn=rr_udf, 
                                   outfref=, 
                                   x_filed_dt=
                                   );

	%put %sysfunc(sasmsg(&smdds,GEN_MAC_START_NOTE,NOQUOTE,&sysmacroname)) ;

	%let successYN=Y;

	/***********************************************/
	/* Load transmitter data into macro variables  */
	/***********************************************/ 

	/* define the scope of efile udf as macro variables before calling */
	/* %ecmrr_efile_fincen_trnsmttr_var.                               */

	data _null_;
		set ecm_db.efile_udf_def end=eof;
		where udf_type_nm in ('VARCHAR' 'CHAR');
		call symput(udf_nm,"");      
	run; 
		
	%ecmrr_efile_fincen_trnsmttr_var(form_type_cd=SARX);

    
	/*************************************/
	/* Define the list of indicators     */
	/*************************************/
	%let indicator_udf_list=;
    proc sql noprint;
	     select distinct quote(strip(format_nm)) into :indicator_udf_list separated by ' ' 
		 from ecm_form.sarx_code_mapping
         where output_cd='' 
		 ;
     
	/*****************************************************/
	/* Define the list of group tag with attributes      */
	/*****************************************************/
	%let group_attr_udf_list=;
    proc sql noprint;
	     select distinct quote(strip(format_nm)) into :group_attr_udf_list separated by ' ' 
		 from ecm_form.sarx_code_mapping
         where input_cd='*' 
		 ;
     

	/**************************************/
	/* Prepare global data                */
	/**************************************/  

	/* 
	  The count of all <Activity> elements in the batch
	 */ 
    %global activityCount;
	%let activityCount=0;
	proc sql noprint;
		 select strip(put(count(distinct rr_rk),best.)) into :activityCount 
		 from &rr_udf_dsn    
		 ;
	%if &ecm_debug=Y %then %put activityCount=&activityCount;
	%if &sqlrc > 4 %then 
	%do;     
		 %put ERROR: cannot get activity count.;
		 %let successYN=N;
		 %goto mExit;
	%end;
	/*
	  The sum of all <TotalSuspiciousAmountText> element amounts recorded in the batch
	*/
	%let totalAmount=0;
		select strip(put(sum(input(udf_value_str,best20.)),best20.)) into :totalAmount
		from &rr_udf_dsn
		where udf_table_nm='RR' 
              and udf_nm = 'X_SAR_INVOLVED_AMT'       
		 ;
	%if &totalAmount=. %then %let totalAmount=0;

	%if &ecm_debug=Y %then %put totalAmount=&totalAmount;
	%if &sqlrc > 4 %then 
	%do;    
		 %put ERROR: cannot get total amount.;
		 %let successYN=N;
		 %goto mExit;
	%end;
	/* 
	   The count of all <Activity> elements in the batch where the <OriginalAttachmentFileName> element 
	   is recorded with a value referencing the file name of the FinCEN SAR attachment
	*/

	%let ActivityAttachmentCount=0;
		select strip(put(count(distinct rr_rk),best.)) into :ActivityAttachmentCount
		from &rr_udf_dsn
		where udf_table_nm='RR' 
              and udf_nm = 'X_SAR_ATTACHMENT_NM'        
		 ;
	%if &ecm_debug=Y %then %put ActivityAttachmentCount=&ActivityAttachmentCount;
	%if &sqlrc > 4 %then 
	%do;    
		 %put ERROR: cannot get ActivityAttachmentCount.;
		 %let successYN=N;
		 %goto mExit;
	%end;
	 
	/* 
	   The count of all <OriginalAttachmentFileName> elements in the batch that contain a value referencing 
	   the file name of the FinCEN SAR attachment
	*/

    %global AttachmentCount;
	%let AttachmentCount=0;
		select strip(put(count(*),best.)) into :AttachmentCount
		from &rr_udf_dsn
		where udf_table_nm='RR' 
              and udf_nm = 'X_SAR_ATTACHMENT_NM'			  
		 ;
	%if &ecm_debug=Y %then %put AttachmentCount=&AttachmentCount;
	%if &sqlrc > 4 %then 
	%do;    
		 %put ERROR: cannot get AttachmentCount.;
		 %let successYN=N;
		 %goto mExit;
	%end;
	 
	/*
	   The count of all <Party> elements in the batch where the <ActivityPartyTypeCode> is equal to “33” (Subject)
	*/
	%let PartyCount=0;
		select strip(put(count(*),best.)) into :PartyCount 
		from (select distinct rr_rk, udf_table_nm , row_no 
			  from &rr_udf_dsn
			  where udf_table_nm in ('X_SUBJECT' )
			  )
		 ;
	%if &ecm_debug=Y %then %put partyCount=&PartyCount;
	%if &sqlrc > 4 %then 
	%do;     
		 %put ERROR: cannot get party count.;
		 %let successYN=N;
		 %goto mExit;
	%end;

	quit;

	/* format filng date */
	%global FilingDate;
	%let FilingDate=%sysfunc(today(),yymmddn8.);
	%if &ecm_debug=Y %then %put FilingDate=&FilingDate;

	/*************************************/
	/* Write UDF names and values as xml */
	/*************************************/
	 
	data _null_;   
	 
	%if %str(&outfref)=%str() %then
	%do; 
		 file "%sysfunc(pathname(work))/sarxRaw.xml";
	%end;
	%else
	%do; 
		 file &outfref lrecl=5000;
	%end;

		 set &rr_udf_dsn end=eof; 
         where udf_nm ne 'X_INST_RK'; /* excluding X_INST_RK because they are saved as inst_key */ 
		 by rr_rk subject_key inst_key udf_table_nm row_no;
		 
		 length stmt $32767;
		 
		 if _n_=1 then link INIT;
		 if first.rr_rk then link RR_INIT;
		 if first.subject_key and subject_key ne ''  then link SUBJECT_INIT;
		 if first.inst_key and inst_key ne ''  then link INST_INIT;
  
         
		 if first.row_no then link ROW_INIT;
		 link node;
		 if last.row_no then link ROW_TERM;  
         if last.inst_key and inst_key ne '' then link INST_TERM;  
		 if last.subject_key and subject_key ne '' then link SUBJECT_TERM;
		 if last.rr_rk then link RR_TERM;
		 if eof then link TERM;
		 
		 return;

	 
	INIT:
		stmt= '<?xml version="1.0" ?>';
		put stmt;
		stmt='<collection>';
		put stmt;
		stmt='<items>';
		put stmt;
		link GLOBAL; 
		link TRANSMITTER; 
	return;

	RR_INIT: 
		stmt='<report>';
		put stmt;
		stmt='<RR_RK>'||strip(put(rr_rk,best.))||'</RR_RK>';
		put stmt;
	return;

	SUBJECT_INIT:
		stmt= '<subject>';
		put stmt;
		stmt='<subject_key>'||strip(subject_key)||'</subject_key>';
		put stmt;
	return;

	INST_INIT:
	    if subject_key > . then
		do;
		    stmt='<subjectInstitution>';
		end;
		else
		do;
			stmt= '<institution>';
		end;
		put stmt;
		stmt='<inst_key>'||strip(inst_key)||'</inst_key>';
		put stmt;
	return;

	ROW_INIT:
	    
		if    udf_table_nm not in ('RR' /* 'X_SUBJECT' */)  
		      /* generate only table tag for fields with multiple check boxes */
		      and ( (not multi_checkbox)  or 
                    (multi_checkbox and first.udf_table_nm and first.row_no)
		           ) 
        then
		do; /* if format exist for the udf table name, add the formatted value as attribute to add to group tag */
	
		    if udf_table_nm in (&group_attr_udf_list) then
			do;
			   stmt= '<'||strip(udf_table_nm)
			         ||' '
                     ||strip(putc('*',udf_table_nm))
                     ||'>';
			end;
			else
			do;
			stmt= '<'||strip(udf_table_nm)||'>';
			end;
			/* Some scenarios with Other Text in middle of choices will cause a new <RR_RK> row to be started. */
			/* Hardcoding to not generate this extra row.                                                      */
			if upcase(udf_nm) = 'X_INST_SF_TYPE_OTH_TXT' and not first.udf_table_nm then 
			   return;
			   
			put stmt;
		
			if udf_table_nm not in ('RR') and not multi_checkbox then
			do;
			    /* Some scenarios with Other Text in middle of choices will cause a new <RR_RK> row to be started. */
			    /* Hardcoding to not generate this extra row.                                                      */
			    if upcase(udf_nm) = 'X_INST_SF_TYPE_OTH_TXT' then do;
				   return;
				   end;
                else do;
				   stmt= '<RR_RK>'||strip(put(rr_rk,best.))||'</RR_RK>';
				   put stmt;
				   stmt= '<ROW_NO>'||strip(put(row_no,best.))||'</ROW_NO>';
				   put stmt;
				   end;
			end;
        
		end;
	return;

	NODE:
		if udf_nm in (&indicator_udf_list )  then
		do;
			stmt= '<'||strip(udf_nm)||'_'||strip(udf_value_str)||'>Y</'||strip(udf_nm)||'_'||strip(htmlencode(udf_value_str))||'>';
			put stmt;
		end;
		else if udf_nm = 'X_SAR_NARRATIVE_DESC' then 
                link LONG_TEXT;
		else if index(udf_nm,'_DTTM') then 
                link DTTM;
		else
		do;
			    if multi_checkbox then
			       stmt= '<checkbox_code>'||strip(htmlencode(udf_value_str))||'</checkbox_code>';
				else  if desc_only then
			       stmt= '<description_text>'||strip(htmlencode(udf_value_str))||'</description_text>';
			    else
				   stmt= '<'||strip(udf_nm)||'>'||strip(htmlencode(udf_value_str))||'</'||strip(udf_nm)||'>';
				put stmt;
		end;
	return;

	ROW_TERM: 
		if   udf_table_nm not in ('RR') 
		    /* generate only table tag for fields with multiple check boxes */	
              and ( (not multi_checkbox)  or 
                    (multi_checkbox and last.udf_table_nm and last.row_no)
                  )
		then
		do;
			/* Some scenarios with Other Text in middle of choices will cause a new <RR_RK> row to be started. */
			/* Hardcoding to not generate this extra row.                                                      */
			if upcase(udf_nm) = 'X_INST_SF_TYPE_OTH_TXT' and not last.udf_table_nm then do;
			   return;
			   end;
			stmt= '</'||strip(udf_table_nm)||'>';
			put stmt;
		end;
	return;

	 
	INST_TERM:
     	if subject_key > . then
		do;
		    stmt='</subjectInstitution>';
		end;
		else
		do;
			stmt= '</institution>';
		end;
		 
		put stmt;
	return;

	 
	SUBJECT_TERM:
		stmt= '</subject>';
		put stmt;
	return;

	RR_TERM:
		stmt= '</report>';
		put stmt;
	return;

	TERM:
		stmt='</items>';
		put stmt;
		stmt='</collection>';
		put stmt;
	return;

	GLOBAL:
		put '<global>';
		put "<FilingDate>%trim(&FilingDate)</FilingDate>";
		put "<TotalAmount>%trim(&totalAmount)</TotalAmount>";
		put "<ActivityCount>%trim(&activityCount)</ActivityCount>";
		put "<PartyCount>%trim(&partyCount)</PartyCount>";
		put "<ActivityAttachmentCount>%trim(&ActivityAttachmentCount)</ActivityAttachmentCount>";
		put "<AttachmentCount>%trim(&AttachmentCount)</AttachmentCount>";
		put '</global>';
	return;


	TRANSMITTER:
		put '<transmitter>';
		stmt="<X_FINCEN_TRNSMTTR_NM>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_NM')))||"</X_FINCEN_TRNSMTTR_NM>";
		put stmt;
		stmt="<X_FINCEN_TRNSMTTR_ADDRESS_TXT>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_ADDRESS_TXT')))||"</X_FINCEN_TRNSMTTR_ADDRESS_TXT>";
		put stmt;
		stmt="<X_FINCEN_TRNSMTTR_CITY_NM>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_CITY_NM')))||"</X_FINCEN_TRNSMTTR_CITY_NM>";
		put stmt;
		stmt="<X_FINCEN_TRNSMTTR_ZIP_CD>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_ZIP_CD')))||"</X_FINCEN_TRNSMTTR_ZIP_CD>";
		put stmt;
		stmt="<X_FINCEN_TRNSMTTR_STATE_CD>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_STATE_CD')))||"</X_FINCEN_TRNSMTTR_STATE_CD>";
		put stmt;
		stmt="<X_FINCEN_TRNSMTTR_COUNTRY_CD>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_COUNTRY_CD')))||"</X_FINCEN_TRNSMTTR_COUNTRY_CD>";
		put stmt;
		stmt="<X_FINCEN_TRNSMTTR_PHONE_NO>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_PHONE_NO')))||"</X_FINCEN_TRNSMTTR_PHONE_NO>";
		put stmt;
		stmt="<X_FINCEN_TRNSMTTR_CONTACT_NM>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_CONTACT_NM')))||"</X_FINCEN_TRNSMTTR_CONTACT_NM>";
		put stmt;
		stmt="<X_FINCEN_TRNSMTTR_TIN_NO>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_TIN_NO')))||"</X_FINCEN_TRNSMTTR_TIN_NO>";
		put stmt;
		stmt="<X_FINCEN_TRNSMTTR_TCC>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_TCC')))||"</X_FINCEN_TRNSMTTR_TCC>"; 
		put stmt;
		put '</transmitter>';
	return;

	LONG_TEXT:
        length curr_desc _text $ 32767;      
	    _sequenceNumber=0;
	    curr_desc=htmlencode(compress(udf_value_str,,'kw'));	     
		do while (curr_desc > '' and _sequenceNumber <= 5); 
	       _text = substrn(curr_desc,1,4000);
		   _sequenceNumber =_sequenceNumber + 1;	
		    curr_desc = substrn(curr_desc,4000+1); 
	       if _text ne ' ' and _sequenceNumber <= 5 then
	       do ;	          
	                         
  			   put '<X_SAR_NARRATIVE_DESC>';
               stmt='<_sequenceNumber>'
					||strip(put(_sequenceNumber,best.)) 
					||'</_sequenceNumber>';
			   put stmt;
			   /* New FinCEN rules on 31jul2018, no white space before or after tags. Want to remove any space between the text tags. */
			   put '<_text>' _text +(-1) '</_text>';        					
			   /*put _text;*/
			   /*put '</_text>';*/		 
			   put '</X_SAR_NARRATIVE_DESC>'; 
	       end;
		 end; 	
	return;

    DTTM:
		_utc_dt=compress(substrn(strip(udf_value_str),1,10),'-');
		_utc_tm=substrn(strip(udf_value_str),12,19);
		stmt='<_utc_dt>'||strip(_utc_dt)||'</_utc_dt>';
		put stmt;
		stmt='<_utc_tm>'||strip(_utc_tm)||'</_utc_tm>';
		put stmt;

	return;

	run;

	%if &syserr > 4 %then 
	%do;     
		 %put ERROR: cannot generate rawXml.;
		 %let successYN=N;
		 %goto mExit;
	%end;



%MEXIT:    
    quit;
    %put %sysfunc(sasmsg(&smdds,GEN_MAC_END_NOTE,NOQUOTE,&sysmacroname)) ;


%mend;
 
