
/***********************************************************************************/
/* Copyright (c) 2014-2018, SAS Institute Inc., Cary, NC, USA, All Rights Reserved */
/*                                                                                 */
/* NAME:          ecmrr_efile_fincen_sdtm                                          */
/* PRODUCT:       Enterprise Case Management                                       */
/* PURPOSE:       Driver program for generating a FINCEN e-file for SDTM           */
/*                (secure direct transfer mode)                                    */ 
/* PARAMETERS:    efile_info_dsn=table containing efile information                */
/*                efile_rk=efile key                                               */
/*                form_type_cd=FINCEN form type code                               */
/*                             SAR for Suspiciously Activity Report                */
/*                             NEWCTR for Currency Transaction Report from 2013    */
/*                             CTRX for Currency Transaction Report from 2018 xml  */
/*                             SARX for Suspiciously Activity Report from 2018 xml */
/*                                                                                 */
/* NOTE:          This process will provide x_sdtm_efile_nm to be saved into       */
/*                efile record.                                                    */
/*                It also removes the existing sdtm file before generating a new   */
/*                one with the new timestamp.                                      */ 
/*                Since x_sdtm_efile_nm is saved when the stored process           */
/*                returns the value to the efile UI, this program should be        */
/*                executed thru the UI only.                                       */
/* SAR efile: BSARST.<yyyymmddhhmmss>.<organizationname>                           */
/* CTR efile: BCTRST.<yyyymmddhhmmss>.<organizationname>                           */
/* SAR attachment zip: : BSARST.<yyyymmddhhmmss>.zip                               */
/* The final zip file:  BSARXST.<yyyymmddhhmmss>.<organizationname>.zip. for sar   */
/* For efiles in XML format:                                                       */
/* CTRX efile: CTRXST.<yyyymmddhhmmss>.<organizationname>.xml                      */
/* SARX efile: SARXST.<yyyymmddhhmmss>.<organizationname>.xml                      */
/* SARX attachment zip: : SARXST.<yyyymmddhhmmss>.zip                               */
/* The final zip file:  SARXST.<yyyymmddhhmmss>.<organizationname>.zip. for sar    */  
/***********************************************************************************/

%macro ecmrr_efile_fincen_sdtm(   efile_info_dsn=efile_output_info
                                , efile_rk= 
                                , form_type_cd=);

%put %sysfunc(sasmsg(&smdds,GEN_MAC_START_NOTE,NOQUOTE,&sysmacroname)) ;
 
                
    %let successYN=Y;
     
    %if %upcase(&&&ecm_sdtm_YN_&form_type_cd) ne Y %then %goto mexit;

    /* Check if there is an existing SDTM file for the efile */
    /* It will be deleted in the later step.                 */
    %let old_sdtm_efile_nm=;

    proc sql noprint;
          select strip(udf_value) into :old_sdtm_efile_nm
          from ecm_db.efile_live as live
               , ecm_db.efile_udf_char_value as udf
          where live.efile_rk=&efile_rk
                and udf.efile_rk=&efile_rk
                and live.valid_from_dttm=udf.valid_from_dttm 
                and udf_nm='X_SDTM_EFILE_NM';

    quit;


    /* Retrive  the generated efile names and locations */
   /*  aand define the SDTM file names and locations.   */ 
    data _null_;
         length sdtm_dttm 
                $ 14;
         length ecm_output_fullpath 
                ecm_dat_nm 
                ecm_zip_nm
                sdtm_output_fullpath
                sdtm_dat_nm 
                sdtm_zip_nm
                sdtm_final_zip_nm
                tmp_zip_fullpath
                sdtm_transmission_id
                fincen_form_nm
                $ 1024
                ;
         set &efile_info_dsn;
         if &efile_rk ne efile_rk then
         do;
            call symput('SUCCCESSYN','N');
            put "EFILE_INFO_DSN data does not match the efile rk";
            return;
         end;   
            
         if "&form_type_cd"='NEWCTR' then fincen_form_nm='CTR';
         else  fincen_form_nm=upcase("&form_type_cd");

         sdtm_dttm=compress(put(output_create_dttm,B8601DT15.),'T'); 
         if "&form_type_cd" in ('CTRX' 'SARX') then
            sdtm_prefix=strip(fincen_form_nm)||'ST';
         else  
            sdtm_prefix='B'||strip(fincen_form_nm)||'ST';

         organization_nm="&&&ecm_sdtm_suffix_&form_type_cd";
         sdtm_output_fullpath="&&&ecm_sdtm_efile_path_&form_type_cd";
      

         ecm_output_fullpath=strip("&ecm_efile_path")||"&slash"||strip(output_path_nm);
         ecm_dat_nm=output_nm;


         
         sdtm_transmission_id=strip(sdtm_prefix)
                               ||'.'
                               ||sdtm_dttm
                               ||'-'
                               ||strip(put(efile_rk,best.))
                               ||'.'
                               ||strip(organization_nm);

          if "&form_type_cd" in ('CTRX' 'SARX') then
             sdtm_dat_nm=strip(sdtm_transmission_id)||'.xml';
          else 
             sdtm_dat_nm=strip(sdtm_transmission_id);
                        
        
         if x_output_zip_nm ne '' then 
         do;
           
            ecm_zip_nm=x_output_zip_nm;

            /* attachment zip does not have the organization name to distinguish it from the final .zip */
            sdtm_zip_nm=strip(sdtm_prefix)
                        ||'.'
                        ||sdtm_dttm
                        ||'-'
                        ||strip(put(efile_rk,best.))
                        ||'.zip'; 
            sdtm_final_zip_nm=strip(sdtm_transmission_id)
                        ||'.zip';
                        
            tmp_zip_fullpath=strip(pathname('WORK'))
                             ||"&slash"
                             ||strip(sdtm_transmission_id)
                         ;
         end;
         else 
         do;
            ecm_zip_nm='';
            sdtm_zip_nm='';
            sdtm_final_zip_nm='';
            tmp_zip_fullpath='';
         end;
         

       

         call symput('ECM_OUTPUT_FULLPATH',strip(ecm_output_fullpath));  
         call symput('SDTM_OUTPUT_FULLPATH',strip(sdtm_output_fullpath));  
         call symput('ECM_ZIP_NM',strip(ecm_zip_nm));
         call symput('ECM_DAT_NM',strip(ecm_dat_nm));
         call symput('SDTM_DAT_NM',strip(sdtm_dat_nm));
         call symput('SDTM_ZIP_NM',strip(sdtm_zip_nm));
         call symput('SDTM_FINAL_ZIP_NM',strip(sdtm_final_zip_nm));
         call symput('TMP_ZIP_FULLPATH',strip(tmp_zip_fullpath));

         link DELETE_OLD;

         return;

    DELETE_OLD:
          
         if  "%trim(&old_sdtm_efile_nm)" ne "" then 
         do;         
          fref='OLDFILE';
          rc=filename(fref, strip(sdtm_output_fullpath)||"&slash"||strip("%trim(&old_sdtm_efile_nm)"));
          if rc = 0 and fexist(fref) then
          rc=fdelete(fref);
          rc=filename(fref);
         end;

    return;     
         
    run;


    %if &syserr > 4 %then 
    %do;
         %let successYN=N;
         %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
         %goto mExit;
    %end;




    /* Check the existance of output directory, if not create it. */
    %let RC=;
    %ecm_mkdir(pathname=%nrbquote(&sdtm_output_fullpath),rcvar=RC,resetYN=N);
    %if &RC ne 1 %then %goto mExit;


    
    
    %if %nrbquote(&ecm_zip_nm) ne %then
    %do; /* if attachment zip exists */

        /* Move the attachment .zip and .dat files to the temporary location */
        /* Then zip it up into a single file for SDTM.                       */ 
        %ecm_mkdir(pathname=%nrbquote(&tmp_zip_fullpath),rcvar=RC,resetYN=Y);

        %if &RC ne 1 %then 
        %do;
           %let successYN=N;
           %goto mExit;
        %end;
        filename ecmzip "&ecm_output_fullpath&slash&ecm_zip_nm";
        filename tmpzip "&tmp_zip_fullpath&slash&sdtm_zip_nm";                 
        %ecm_copyfile(infref=ecmzip,outfref=tmpzip,rcvar=RC);

        %if &RC ne 1 %then 
        %do;
           %let successYN=N;
           %goto mExit;
        %end;
         
        filename ecmzip clear;
        filename tmpzip clear;
         
        filename ecmdat "&ecm_output_fullpath&slash&ecm_dat_nm";
        filename tmpdat "&tmp_zip_fullpath&slash&sdtm_dat_nm";
        
                 
        %ecm_copyfile(infref=ecmdat,outfref=tmpdat,rcvar=RC);

        filename ecmdat clear;
        filename tmpdat clear;
        
        %if &RC ne 1 %then 
        %do;
           %let successYN=N;
           %goto mExit;
        %end;

        filename sdtmzip "&sdtm_output_fullpath&slash&sdtm_final_zip_nm";
        %ecm_zip(zippath=%superq(tmp_zip_fullpath),rcvar=RC,outfref=sdtmzip);

        %if %sysfunc(fexist(sdtmzip))=0 %then 
        %do;
              %put "&sdtm_zip_nm is not generated. Check log for error.";
              %goto mExit;
        %end;
        filename sdtmzip clear;
        proc sql noprint;
             update &efile_info_dsn
             set   x_sdtm_efile_nm="&sdtm_final_zip_nm"
                
             ;
        quit;
        

    %end; /* if attachment zip exists */
    %else 
    %do; /* there is no attachment zip */

        /* Just copy the .dat file to the SDTM location and rename it */
        filename ecmdat "&ecm_output_fullpath&slash&ecm_dat_nm";
        filename sdtmdat "&sdtm_output_fullpath&slash&sdtm_dat_nm";  
        %ecm_copyfile(infref=ecmdat,outfref=sdtmdat,rcvar=RC);
      
        filename ecmdat clear;
        filename sdtmdat clear;
 
        %if &RC ne 1 %then 
        %do;
              %put &sdtm_dat_nm is not generated. Check log for error.;
              %goto mExit;
        %end;
            
        proc sql noprint;
             update &efile_info_dsn
             set   x_sdtm_efile_nm="&sdtm_dat_nm"
                
             ;
        quit;



      
    %end; /* there is no attachment zip */
     


%mexit:

%put %sysfunc(sasmsg(&smdds,GEN_MAC_END_NOTE,NOQUOTE,&sysmacroname)) ;

%mend;
