﻿/***********************************************************************************/
/* Copyright (c) 2012-2013, SAS Institute Inc., Cary, NC, USA, All Rights Reserved */
/*                                                                                 */
/* NAME:          ecm_wip_get_attacment                                            */
/* PRODUCT:       Enterprise Case Management                                       */
/* PURPOSE:       extract attachments from attachment services                     */
/* PARAMETERS:                                                                     */
/*                object_type_id: attachment service object type ID. ECM objects   */
/*                                are 54500<i>                                     */
/*                object_id: object ID, usually ECM object RK                      */
/*                outfref: file reference of the path for the output               */
/*                attachment_nm: file name of the attachment                       */
/*                rcvar: variable name for the return code                         */ 
/* NOTE:                                                                           */
/***********************************************************************************/
%macro ecm_wip_get_attachment(object_type_id=,object_id=,outfref=,attachment_nm=,rcvar=RC);

  %put %sysfunc(sasmsg(&smdds,GEN_MAC_START_NOTE,NOQUOTE,&sysmacroname)) ;

  %let &rcvar=0;

  /*
  format of attachment service url:
  http://<hostname:port>/SASWIPClientAccess/rest/content/{object-type-id}/{object-id}/attachments
  */
  /* retreive attachment service url from SAS metadata server if it is not assigned */
  %if not %symexist(ecm_wip_attach_url) %then %global ecm_wip_attach_url;
  %if %superq(ecm_wip_attach_url) = %str()  %then
  %do;
    %if not %symexist(service_endpoint) %then %global service_endpoint;
    %svcreg(type=REST, name=AttachmentService REST);
    %let ecm_wip_attach_url=&service_endpoint;
  %end;

  /* translate  {object-type-id} and {object-id} of the url with the input object type ID and object ID */
  %let request_url=%sysfunc(tranwrd(&ecm_wip_attach_url,%str({object-type-id}),&object_type_id));
  %let request_url=%sysfunc(tranwrd(&request_url,%str({object-id}),&object_id));

  /* call attachment service to get the list of attachments */
  filename srv_resp temp;
  libname  srv_resp xml92  xmlmap="&ecm_sasmisc_path/xml/map/AttachmentClient.map";

  %ecm_wip_get_attachment_http_req(request_url=&request_url,outfile=srv_resp);

  %if not %sysfunc(exist(SRV_RESP.ATTACHMENT)) %then
  %do;
    %let &rcvar=8;
    %put %sysfunc(sasmsg(&smdds,GEN_TABLE_MISSING_ERROR,NOQUOTE,SRV_RESP.ATTACHMENT)) ;
    %goto mExit;
  %end;

  /* find the attachment ID of the most recent document with the input attachment name */
  %let attachment_id=;
  proc sql noprint;
    select strip(put(max(attachment_id),best.)) into :attachment_id
        from srv_resp.attachment       
        where attachment_nm="&attachment_nm" ;
  quit;

  %if &syserr > 4 %then 
  %do;
    %let &rcvar=&syserr;
    %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
    %goto mExit;
  %end;

  /* retreive the content of the attachment by attachment ID */

  libname srv_resp clear; 
  %if &attachment_id ne and &attachment_id ne . %then
  %do;
    %ecm_wip_get_attachment_http_req(request_url=&request_url/%trim(&attachment_id)/contents,outfile=&outfref);
       
    %let &rcvar=&syserr; 
    %if &syserr > 4 %then 
    %do;
      %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
      %goto mExit;
    %end;
  %end;
  %else
  %do;
    %let &rcvar=8;
    %put %sysfunc(sasmsg(&smdds,SAR_EFILE_ATTACH_ID_MISSING_ERROR,NOQUOTE,&attachment_nm)) ;
    %goto mExit;
  %end; 
 
%mExit: 

  %put %sysfunc(sasmsg(&smdds,GEN_MAC_END_NOTE,NOQUOTE,&sysmacroname)) ;

%mend;

%macro ecm_wip_get_attachment_http_req(request_url=,outfile=);
  %put NOTE: HTTP request to &request_url;

  %if not %symexist(ecm_wip_attach_cookie) %then %global ecm_wip_attach_cookie;

  %if %str(&ecm_wip_attach_cookie) ne %str() %then
  %do;
  /* If cookie macro not empty create header file containing cookie value */
  filename hdrin temp;
  data _null_;
    file hdrin;
    put "Cookie: &ecm_wip_attach_cookie";
  run;
  %end;

  filename hdrout temp;

  proc http url="&request_url"
            out=&outfile
            method="get"
            ct="application/x-www-form-urlencoded" 
  %if %str(&ecm_wip_attach_cookie) eq %str() %then
  %do;
            HTTP_TOKENAUTH /* new in 9.4 to support one-time password */
  %end;
  %else
  %do;
            headerin=hdrin
  %end;
            headerout=hdrout
            headerout_overwrite
  %if %str(&ecm_webusername) ne %str() and
      %str(&ecm_webpassword) ne %str() %then
  %do;
            webusername="&ecm_webusername" webpassword="&ecm_webpassword"  
  %end;    
            ;
  run;

  %local httpstatus message;
  data _null_;
    infile hdrout scanover truncover;
    input @'HTTP/1.1' code 4. message $255.;
    call symputx('httpstatus',code);
    call symput('message',trim(message));
    stop;
  run;

  %if &httpstatus eq 401 and %str(&ecm_wip_attach_cookie) ne %str() %then
  %do;
    /* Request made with cookie failed with 401 Unauthorized; session expired?
       Retry the request with tokenauth, resetting cookie macro to get the new cookie extracted */
    %put NOTE: Retrying request following unauthorized response possibly indicating session expired;
    %let ecm_wip_attach_cookie=;
    proc http url="&request_url"
              out=&outfile
              method="get"
              ct="application/x-www-form-urlencoded" 
              HTTP_TOKENAUTH /* new in 9.4 to support one-time password */
              headerout=hdrout
              headerout_overwrite
    %if %str(&ecm_webusername) ne %str() and
        %str(&ecm_webpassword) ne %str() %then
    %do;
              webusername="&ecm_webusername" webpassword="&ecm_webpassword"  
    %end;    
            ;
    run;

    data _null_;
      infile hdrout scanover truncover;
      input @'HTTP/1.1' code 4. message $255.;
      call symputx('httpstatus',code);
      call symput('message',trim(message));
      stop;
    run;
    %if &httpstatus ne 200 %then
    %do;
      %put ERROR: Retry request following unauthorized response failed with HTTP status &httpstatus &message;
      %goto mExit;
    %end;
  %end;

  %if %str(&ecm_wip_attach_cookie) eq %str() %then
  %do;
    /* Request made without cookie => new session and therefore we need to process header for new session cookie */
    data _null_;
      length hdrvalue $255;
      infile hdrout delimiter=':';
      input @;
      if ( index(_infile_,'Set-Cookie') > 0 ) then do;
        input @'Set-Cookie' hdrvalue;
        cookie = substrn(hdrvalue, 0, find(hdrvalue,';',1));
        call symput('ecm_wip_attach_cookie',trim(cookie));
        stop;
      end;
    run;
  %end;
%mExit:
%mend;
