
/***********************************************************************************/
/* Copyright (c) 2017-2018, SAS Institute Inc., Cary, NC, USA, All Rights Reserved */
/*                                                                                 */
/* NAME:          ecmrr_efile_fincen_ctrx_sum                                      */
/* PRODUCT:       Enterprise Case Management                                       */
/* PURPOSE:       Convert udf name and value pairs into xml                        */
/* PARAMETERS:                                                                     */
/*                rr_udf_dsn: name of dataset with selected report data            */
/*                outfref: file reference of the output xml                        */
/*                x_filed_dt: date used as filing date                             */
/***********************************************************************************/

%macro ecmrr_efile_fincen_ctrx_sum(rr_udf_dsn=rr_udf, 
                                   outfref=, 
                                   x_filed_dt=
                                   );

%put %sysfunc(sasmsg(&smdds,GEN_MAC_START_NOTE,NOQUOTE,&sysmacroname)) ;

%let successYN=Y;

/***********************************************/
/* Load transmitter data into macro variables  */
/***********************************************/ 

/* define the scope of efile udf as macro variables before calling */
/* %ecmrr_efile_fincen_trnsmttr_var.                               */

data _null_;
    set ecm_db.efile_udf_def end=eof;
    where udf_type_nm in ('VARCHAR' 'CHAR');
    call symput(udf_nm,"");      
run; 
    
%ecmrr_efile_fincen_trnsmttr_var(form_type_cd=CTRX);

/**************************************/
/* Prepare global data                */
/**************************************/  

/* count number of CTRX reports */ 
%let activityCount=0;
proc sql noprint;
     select strip(put(count(distinct rr_rk),best.)) into :activityCount 
     from &rr_udf_dsn    
     ;
%if &ecm_debug=Y %then %put activityCount=&activityCount;
%if &sqlrc > 4 %then 
%do;     
     %put ERROR: cannot get activity count.;
     %let successYN=N;
     %goto mExit;
%end;

/* get cash in and out amount total */
%let totalAmount=0;
    select strip(put(sum(input(udf_value_str,best.)),best.)) into :totalAmount
    from &rr_udf_dsn
    where (udf_table_nm='RR' and udf_nm in ('X_US_CASH_IN_AMT' 'X_US_CASH_OUT_AMT'))
          or (udf_table_nm in ('X_FOREIGN_CASH_IN' 'X_FOREIGN_CASH_OUT') 
          )         
     ;
%if &ecm_debug=Y %then %put totalAmount=&totalAmount;
%if &sqlrc > 4 %then 
%do;    
     %put ERROR: cannot get total amount.;
     %let successYN=N;
     %goto mExit;
%end;

/* count number of parties including transmitter, transmitter contact,    */
/* filing institution, filing institution contact, branches, and subjects */
%let PartyCount=0;
    select strip(put(count(*)+&activityCount * 4,best.)) into :PartyCount 
    from (select distinct rr_rk, udf_table_nm , row_no 
          from &rr_udf_dsn
          where udf_table_nm in ('X_INSTITUTION' 'X_PERSON_INVOLVED' )
          )
     ;
%if &ecm_debug=Y %then %put partyCount=&PartyCount;
%if &sqlrc > 4 %then 
%do;     
     %put ERROR: cannot get party count.;
     %let successYN=N;
     %goto mExit;
%end;

quit;

/* format filng date */
%let FilingDate=%sysfunc(today(),yymmddn8.);
%if &ecm_debug=Y %then %put FilingDate=&FilingDate;

/*************************************/
/* Write UDF names and values as xml */
/*************************************/
data _null_;
%if %str(&outfref)=%str() %then
%do; 
     file "%sysfunc(pathname(work))/ctrxRaw.xml";
%end;
%else
%do; 
     file &outfref;
%end;

     set &rr_udf_dsn end=eof;   
     by rr_rk subject_key udf_table_nm row_no;
     
     length stmt $1024;
     if _n_=1 then link INIT;
     if first.rr_rk then link RR_INIT;
     if first.subject_key and subject_key ne ''  then link SUBJECT_INIT;      
     if first.row_no then link ROW_INIT;
     link node;
     if last.row_no then link ROW_TERM;     
     if last.subject_key and subject_key ne '' then link SUBJECT_TERM;
     if last.rr_rk then link RR_TERM;
     if eof then link TERM;
     
     return;

INIT:
    stmt= '<?xml version="1.0" ?>';
    put stmt;
    stmt='<collection>';
    put stmt;
    stmt='<items>';
    put stmt;
    link GLOBAL; 
    link TRANSMITTER; 
return;

RR_INIT: 
    stmt='<report>';
    put stmt;
    stmt='<RR_RK>'||strip(put(rr_rk,best.))||'</RR_RK>';
    put stmt;
return;

SUBJECT_INIT:
    stmt= '<subject>';
    put stmt;
    stmt='<subject_key>'||strip(subject_key)||'</subject_key>';
    put stmt;
return;

ROW_INIT:
 
    if udf_table_nm not in ('RR' 'X_PERSON_INVOLVED') then
    do;
        stmt= '<'||strip(udf_table_nm)||'>';
        put stmt;
    end;
    if udf_table_nm not in ('RR') then
    do;
        stmt= '<RR_RK>'||strip(put(rr_rk,best.))||'</RR_RK>';
        put stmt;
        stmt= '<ROW_NO>'||strip(put(row_no,best.))||'</ROW_NO>';
        put stmt;
    end;
return;

NODE:
    if udf_nm in ('X_TRAN_METHOD_TYPE_CD' 'X_CTR_FILING_TYPE_CD')  then
    do;
        stmt= '<'||strip(udf_nm)||'_'||strip(udf_value_str)||'>Y</'||strip(udf_nm)||'_'||strip(htmlencode(udf_value_str))||'>';
        put stmt;
    end;
    else
    do;
        stmt= '<'||strip(udf_nm)||'>'||strip(htmlencode(udf_value_str))||'</'||strip(udf_nm)||'>';
        put stmt;
    end;
return;

ROW_TERM: 
    if udf_table_nm not in ('RR' 'X_PERSON_INVOLVED') then
    do;
        stmt= '</'||strip(udf_table_nm)||'>';
        put stmt;
    end;
return;

 
SUBJECT_TERM:
    stmt= '</subject>';
    put stmt;
return;

RR_TERM:
    stmt= '</report>';
    put stmt;
return;

TERM:
    stmt='</items>';
    put stmt;
    stmt='</collection>';
    put stmt;
return;

GLOBAL:
    put '<global>';
    put "<FilingDate>%trim(&FilingDate)</FilingDate>";
    put "<TotalAmount>%trim(&totalAmount)</TotalAmount>";
    put "<ActivityCount>%trim(&activityCount)</ActivityCount>";
    put "<PartyCount>%trim(&partyCount)</PartyCount>";
    put '</global>';
return;


TRANSMITTER:
    put '<transmitter>';
    stmt="<X_FINCEN_TRNSMTTR_NM>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_NM')))||"</X_FINCEN_TRNSMTTR_NM>";
    put stmt;
    stmt="<X_FINCEN_TRNSMTTR_ADDRESS_TXT>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_ADDRESS_TXT')))||"</X_FINCEN_TRNSMTTR_ADDRESS_TXT>";
    put stmt;
    stmt="<X_FINCEN_TRNSMTTR_CITY_NM>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_CITY_NM')))||"</X_FINCEN_TRNSMTTR_CITY_NM>";
    put stmt;
    stmt="<X_FINCEN_TRNSMTTR_ZIP_CD>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_ZIP_CD')))||"</X_FINCEN_TRNSMTTR_ZIP_CD>";
    put stmt;
    stmt="<X_FINCEN_TRNSMTTR_STATE_CD>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_STATE_CD')))||"</X_FINCEN_TRNSMTTR_STATE_CD>";
    put stmt;
    stmt="<X_FINCEN_TRNSMTTR_COUNTRY_CD>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_COUNTRY_CD')))||"</X_FINCEN_TRNSMTTR_COUNTRY_CD>";
    put stmt;
    stmt="<X_FINCEN_TRNSMTTR_PHONE_NO>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_PHONE_NO')))||"</X_FINCEN_TRNSMTTR_PHONE_NO>";
    put stmt;
    stmt="<X_FINCEN_TRNSMTTR_CONTACT_NM>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_CONTACT_NM')))||"</X_FINCEN_TRNSMTTR_CONTACT_NM>";
    put stmt;
    stmt="<X_FINCEN_TRNSMTTR_TIN_NO>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_TIN_NO')))||"</X_FINCEN_TRNSMTTR_TIN_NO>";
    put stmt;
    stmt="<X_FINCEN_TRNSMTTR_TCC>"||strip(htmlencode(symgetc('X_FINCEN_TRNSMTTR_TCC')))||"</X_FINCEN_TRNSMTTR_TCC>"; 
    put stmt;
    put '</transmitter>';
return;
run;

%if &syserr > 4 %then 
%do;     
     %put ERROR: cannot generate rawXml.;
     %let successYN=N;
     %goto mExit;
%end;



%MEXIT:    

    %put %sysfunc(sasmsg(&smdds,GEN_MAC_END_NOTE,NOQUOTE,&sysmacroname)) ;


%mend;
 
