/***********************************************************************************/
/* Copyright (c) 2017, SAS Institute Inc., Cary, NC, USA, All Rights Reserved      */
/*                                                                                 */
/* NAME:          ecmrr_resp_fincen_ctrx_read                                      */
/* PRODUCT:       Enterprise Case Management                                       */
/* PURPOSE:       read the CTRX acknowledgement receipt XML with XML map           */
/* PARAMETERS:    INFREF=fileref pointing to a CTRX repsonse file.                 */
/* NOTE:          Output tables are:                                               */
/*                _1A: efile level response                                        */
/*                _3A: rr (aka activity) level response                            */
/*                _ctrx_error_codes: error level response                          */
/*                The structures of the tables were originally designed based on   */
/*                the old .asc response files structures.                          */
/*                They will be used as input to ecmrr_resp_fincen_update           */ 
/***********************************************************************************/


%macro ecmrr_resp_fincen_ctrx_read ( infref= 
                                     );
     %put %sysfunc(sasmsg(&smdds,GEN_MAC_START_NOTE,NOQUOTE,&sysmacroname)) ;


    /* Remove output from previous run */
    proc datasets lib=work nowarn nolist;
         delete 
                _1A
                _3A 
                _ctrx_error_codes
				efilingactivityxml
				efilingbatchxml
				efilingactivityerrorxml
				efilingsubmissionxml
    ;
    quit;

 
	/* load the input file to sas library with xml maper. */
    libname &infref xml92 xmlmap="%sysfunc(pathname(ecm_form))&slash.ctrx_fincen_ackd.map";

    /* Make the xml mapped data into sas tables before releasing the library*/
    proc copy in=&infref out=work;
    run;

	%if &syserr > 4 %then 
    %do;
	    libname &infref clear;
        %let successYN=N;
		%put Cannot load xml data into sas table.; 
        %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
        %goto mExit;
    %end;
    %if %upcase(&ecm_debug) = N %then
	%do;
	libname &infref clear;
    %end;
     proc sql;
	      /* Read the efile level data to create _1A */
	      /* In the xml output, coverage period is not relevant any more */
	      create table _1A as
		  select EfilingSubmissionXML_StatusCode as SUBMISSION_STATUS_CD,
		         '' as RECORD_TYPE length=2,
                 '' as COVERAGE_BEGIN_DT length=8,
                 '' as COVERAGE_END_DT length=8 
		  from EfilingSubmissionXml		       
		  ;
		  %if &sqlrc > 4 %then 
   		  %do;
        	  %let successYN=N;
              %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
              %goto mExit;
          %end;

		  /* Read the activtiy level data to create _3A              */
	      /* In the xml output, record_type is not relevant any more */
		  /* SeqNum contains the RR_RK. Use it as _3A USER_FIELD     */        

	      create table _3A as
		  select EFilingActivityXML_ORDINAL as TRANS_SEQ_NO,
		         '' as RECORD_TYPE length=2,
				 put(EFilingActivityXML_SeqNum,z10.) as USER_FIELD,
			     BSAID as BSA_ID length=14  
		  from EfilingActivityXml		       
		  ;
		  %if &sqlrc > 4 %then 
   		  %do;
        	  %let successYN=N;
              %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
              %goto mExit;
          %end;

		  /* Read error level data to create _CTRX_error_codes                         */
	      /* In the xml output, record_type and line_no are not relevant any more      */
		  /* ErrorContext contains xml path of the data value that trigger the error   */ 

		  create table _ctrx_error_codes as
		  select err.ErrorTypeCode as ERROR_CODE,
                 err.EFilingActivityXML_ORDINAL as LINE_NO, 
				 '' as RECORD_TYPE length=2,
				 err.Efilingactivityerrorxml_SeqNum as ERROR_SEQ_NO,
                 err.EFilingActivityXML_ORDINAL as TRANS_SEQ_NO,
		         act.user_field,
				 act.bsa_id,
				 err.ErrorContextText as ERROR_CONTEXT			     
		  from EfilingActivityErrorXml as err
          left join _3A as act
          on  err.EFilingActivityXML_ORDINAL = act.trans_seq_no 
		  where error_code ne ''
		  ;
		  %if &sqlrc > 4 %then 
   		  %do;
        	  %let successYN=N;
              %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
              %goto mExit;
          %end;

      quit;    

%mexit:

 %put %sysfunc(sasmsg(&smdds,GEN_MAC_END_NOTE,NOQUOTE,&sysmacroname)) ;


%mend;
