/***********************************************************************************/
/* Copyright (c) 2014-2018, SAS Institute Inc., Cary, NC, USA, All Rights Reserved */
/*                                                                                 */
/* NAME:        ecmrr_resp_fincen_report                                           */
/* PRODUCT:     Enterprise Case Management                                         */
/* PURPOSE:     Process the uploaded file which is associated with an efile.       */
/* PARAMETERS:  INPUT_TYPE=type of input parameter.                                */
/*                valid values are; FROM_DTTM, RESPONSE_ID, EFILE_RK, RR_RK, BSA_ID*/ 
/*                If it is blank, all response data will be reported.              */ 
/*              FROM_DTTM=filter the response data by this SAS timestamp           */
/*              RESPONSE_ID=filter the response data by this response ID           */
/*              EFILE_RK  = filter the response data by this efile key             */
/*              RR_RK     = filter the response data by this report key            */ 
/*              BSA_ID    = filter the response data by this BSA ID                */
/*              RESPONSE_TYPE_CD=asc for acknowledgement receipt of sar, ctr, doep */
/*              FORM_CONFIG_RK=form key (201 for SAR and 202 for NEWCTR.           */
/*                  This is a required parameter.                                  */
/*              PRINTYN=Y to generate report.                                      */
/*                      N to prepare report data only.                             */
/* NOTE:       CTRX and SARX response type is XML.                                 */
/*             Since ecmrr_run_resp_fincen_report stored process hardcode          */
/*             response_type_cd as ASC, response_type_cd will change to XML when   */
/*             the form is CTRX/SARX (form_config_rk=212/211)                      */
/***********************************************************************************/


%macro ecmrr_resp_fincen_report(   input_type=
                                 , from_dttm=
                                 , response_id=
                                 , efile_rk=
                                 , rr_rk=
                                 , bsa_id=
                                 , response_type_cd=ASC
                                 , form_config_rk=
                                 , printYN=Y
                               
                             );

    %put %sysfunc(sasmsg(&smdds,GEN_MAC_START_NOTE,NOQUOTE,&sysmacroname)) ;


    %let successYN=Y;

	/* fix response_type_cd when the form is CTRX */
	%if ((&form_config_rk=212) or (&form_config_rk=211)) and (%upcase(&response_type_cd)=ASC) %then
	%do;
	    %let response_type_cd=XML;
	%end;
 
    %local filter_response_id filter_efile_rk filter_rr_rk form_type_cd error_code_table;

    %ecm_create_ref_fmt(ref_table_list="RT_RR_STATUS" "RT_EFILE_STATUS" ); 
    proc format;
         value $respErrCode 
         'P'='Primary'
         'S'='Secondary'
		 'F'='Fatal'
         ;
    run;


     %if &successYN=N %then %goto mEXIT; 

    /* Get the form type code. e.g. SAR or NEWCTR */
    proc sql noprint;
         select strip(form_type_cd) 
                into
                :form_type_cd 
         from ecm_db.form_config
         where form_config_rk=&form_config_rk
    ;
    %if &sqlrc > 4 %then 
    %do;
        %let successYN=N;
        %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
        %goto mExit;
    %end;


    quit;

    /* Define the error code table name based on the form type code */
    %let error_code_table=ecm_form.%trim(%left(&form_type_cd))_fincen_error_code;

    /* Initialize filter lists */
    %let filter_response_id=;
    %let filter_efile_rk=;
    %let filter_rr_rk=;


    /* If no input type is specified, all response data for the form will be reported */
    %if %upcase(&input_type)=%str() %then
    %do;
            proc sql noprint;
            select distinct quote(strip(response_id)) 
                   into :filter_response_id
                   separated by ' ' 
            from  ecm_rpt.ecmrr_resp_fincen_summary
            where  form_config_rk=&form_config_rk
             %if %str(&response_type_cd) ne %str() %then 
             %do;
                   and upcase(response_type_cd)="%upcase(&response_type_cd)"
             %end; 

            %if &sqlrc > 4 %then 
            %do;
            %let successYN=N;
            %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
            %goto mExit;
            %end;


                  
            ;  
                
    %end;

    /* Define the list of response ID of the response files which are processed after the FROM_DTTM time stamp */

    %if %upcase(&input_type)=FROM_DTTM %then
    %do;
        %if %str(&from_dttm) ne %str() %then
        %do; 
            %let filter_response_id=;
            proc sql noprint;
            select distinct quote(strip(response_id))
            into :filter_response_id
                  separated by ' '
            from  ecm_rpt.ecmrr_resp_fincen_summary
            where form_config_rk=&form_config_rk
                  and processed_dttm >= &from_dttm
            ;
            %if &sqlrc > 4 %then 
            %do;
                 %let successYN=N;
                 %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
                 %goto mExit;
            %end;               
        %end;
    %end;


    %else %if %upcase(&input_type)=RESPONSE_ID %then
    %do;
        %if "%nrbquote(&response_id)" ne %nrbquote() %then
        %do; 
            %let filter_response_id="&response_id";
            %let filter_efile_rk=;
            %let filter_rr_rk=;     
        %end;
    %end;

    %else %if %upcase(&input_type)=EFILE_RK %then
    %do;
        %if %str(&efile_rk) ne %str() %then
        %do; 
            %let filter_rr_rk=;
            %let filter_response_id=;
            %let filter_efile_rk=&efile_rk;
 
             /* Find the IDs of the response files that contain the efile */
             /* If the process is done right, there should only be one response file per an efile. */
             /* However, the same efile may be submitted multiple times during testing.   */
             proc sql noprint;
             select  distinct quote(strip(response_id))               
             into   :filter_response_id   
                     separated by ' '    
             from ecm_rpt.ecmrr_resp_fincen_summary
             where efile_rk=&efile_rk
              %if %str(&response_type_cd) ne %str() %then 
             %do;
                   and upcase(response_type_cd)="%upcase(&response_type_cd)"
             %end; 
     
            ; 
        %if &sqlrc > 4 %then 
        %do;
           %let successYN=N;
           %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
           %goto mExit;
        %end;


        
        %end;
    %end;

    %else %if %upcase(&input_type)=RR_RK  %then
    %do;
        %if %str(&rr_rk) ne %str() %then
        %do; 
            %let filter_rr_rk=&rr_rk;
            %let filter_response_id=;
            %let filter_efile_rk=;
             /* Find the IDs of the response files that contain the report */
             /* If the process is done right, there should only be one response file per report */
             /* However, more response files may be created for the same report during testing.   */
             proc sql noprint ;
             select distinct  quote(strip(rpt.response_id)) 
                    , put(rpt.efile_rk,best.)
             into   :filter_response_id
                  , :filter_efile_rk 
             separated by ' '
             from   ecm_rpt.ecmrr_resp_fincen_report_list as rpt
                  , ecm_rpt.ecmrr_resp_fincen_summary as sum
             where rpt.rr_rk=&rr_rk
                   and rpt.response_id=sum.response_id
             %if %str(&response_type_cd) ne %str() %then 
             %do;
                  and upcase(sum.response_type_cd)="%upcase(&response_type_cd)"
             %end; 
             
        ; 
            %if &sqlrc > 4 %then 
            %do;
                %let successYN=N;
                %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
                %goto mExit;
            %end;


        
        %end;
    %end;

    %else %if %upcase(&input_type)=BSA_ID  %then
    %do;
        %if %str(&bsa_id) ne %str() %then
        %do; 
             /* Find the ID of the response files that has reports with the same BSA_ID */
             /* Multiple reports may have the same BSA_ID when they are for correction or update */
           
            %let filter_rr_rk=;
            %let filter_response_id=;
            %let filter_efile_rk=;
            proc sql noprint;
             select distinct quote(strip(rpt.response_id)) 
                    , put(rpt.efile_rk,best.)
                    , put(rpt.rr_rk,best.) 
             into   :filter_response_id
                  , :filter_efile_rk
                  , :filter_rr_rk
             separated by ' '
             from   ecm_rpt.ecmrr_resp_fincen_report_list as rpt
                  , ecm_rpt.ecmrr_resp_fincen_summary as sum
             where rpt.bsa_id="&bsa_id"
                   and rpt.response_id=sum.response_id
             %if %str(&response_type_cd) ne %str() %then 
             %do;
                  and upcase(sum.response_type_cd)="%upcase(&response_type_cd)"
             %end; 
     
             ; 
             %if &sqlrc > 4 %then 
             %do;
                 %let successYN=N;
                 %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
                 %goto mExit;
             %end;
        
        %end;
    %end;


   
    proc sql noprint;

          /* subset the RESP_FINCEN_SUMMARY with the list of response IDs */
         create table resp_fincen_summary_print as
         select   sum.*
                , ef.efile_id
         from   ecm_rpt.ecmrr_resp_fincen_summary as sum
         left join ecm_db.efile_live as ef
         on   sum.efile_rk=ef.efile_rk
         where 1
         %if %str(&filter_response_id) ne %str() %then
         %do;
              and sum.response_id in (&filter_response_id)
         %end;
              
        ;

        %if &sqlrc > 4 %then 
        %do;
            %let successYN=N;
            %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
           %goto mExit;
        %end;


        
         /* subset the RESP_FINCEN_REPORT_LIST with the list of RR_RK */

         create table resp_fincen_report_print as
         select   rpt.*
                , rr.rr_id
         from   ecm_rpt.ecmrr_resp_fincen_report_list as rpt
         left join ecm_db.rr_live as rr
         on  rpt.rr_rk=rr.rr_rk
         where 1
         %if %str(&filter_response_id) ne %str() %then
         %do;
               and rpt.response_id in (&filter_response_id)
         %end;
         

         %if %str(&filter_efile_rk) ne %str() %then
         %do;
                and rpt.efile_rk in (&filter_efile_rk)
          %end;      
          %if %str(&filter_rr_rk) ne %str() %then 
          %do;
                and rpt.rr_rk in (&filter_rr_rk)
          %end;
          order by response_id, trans_seq_no
    ;
         
          %if &sqlrc > 4 %then 
          %do;
              %let successYN=N;
              %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
              %goto mExit;
          %end;

         /* subset the RESP_FINCEN_ERROR_LIST with the list of Reponse ID, EFILE_RK or RR_RK */


         create table resp_fincen_error_print as
         select a.*,
                rr.rr_id
         from
         (
         select     err.*
                  , ref.error_desc
				  %if %upcase(&response_type_cd) = XML %then
				  %do;
				    , ref.field_reference
				  %end;

         from      ecm_rpt.ecmrr_resp_fincen_error_list as err
         left join &error_code_table as ref
         on  err.error_code=ref.error_cd
             and err.record_type=ref.error_record_type
         where 1 
         %if %str(&filter_response_id) ne %str() %then
         %do;
                and err.response_id in (&filter_response_id)
         %end;
         %if %str(&filter_efile_rk) ne %str() %then
         %do;
                and err.efile_rk in (&filter_efile_rk)
          %end;      
          %if %str(&filter_rr_rk) ne %str() %then 
          %do;
                and err.rr_rk in (&filter_rr_rk)
          %end;
          ) as a
          left join ecm_db.rr_live as rr
          on a.rr_rk=rr.rr_rk
          order by response_id, trans_seq_no, line_no, error_seq_no
    ;

          %if &sqlrc > 4 %then 
         %do;
            %let successYN=N;
            %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
            %goto mExit;
         %end;


    quit;

  
  

%if %upcase(&printYN)=Y %then 
%do;
    title;
    %if %str(&filter_response_id)=%str() %then
    %do;
        title; 
        data _null_;
             file print notitles;
             put "No %trim(&form_type_cd) Response Found.";
        run;
        %goto mExit;
    %end;

     %let response_msg_found=0;

         /* Check if there is any processing messages to be displayed */
         proc sql noprint;
         select '1'  
                into :response_msg_found

         from resp_fincen_summary_print  
         where response_processing_msg ne ''
         ; 
    quit;

    proc sql noprint;
         create view resp_fincen_summary_view as
         select   resp.*
                , ef.efile_status_cd  
         from resp_fincen_summary_print as resp 
         left join ecm_db.efile_live as ef
         on resp.efile_rk=ef.efile_rk
         ; 
  
         create view resp_fincen_report_view as
         select   resp.*
                , rept.rr_agency_ref_id
                , rept.rr_status_cd
         from resp_fincen_report_print as resp 
         left join ecm_db.rr_live as rept
         on resp.rr_rk=rept.rr_rk
         ; 

    proc report data=resp_fincen_summary_view nowindows center split='*'  ;
         column response_id  
                efile_id 
                efile_status_cd
				%if %upcase(&response_type_cd) = XML %then
				%do;
                submission_status_cd
                %end; 
                record_3A_cnt 
                primary_error_cnt secondary_error_cnt processed_dttm
         %if %str(&response_msg_found) = %str(1) %then
         %do;
              response_processing_msg
         %end; 
          ;
         define response_id /width=30 flow;
         define efile_id /width=10 flow "E-File ID";
         define efile_status_cd /width=10 flow "E-File Status" format=$RT_EFILE_STATUS.;
         %if %str(&response_msg_found) = %str(1) %then
         %do;
         define response_processing_msg /flow width=30;
         %end;
		 %if %upcase(&response_type_cd) = XML %then
         %do;
		 define record_3A_cnt /"Number of*Activities";
         define submission_status_cd /"Submission Status Code";
         %end;
         title "&form_type_cd Response Summary";      
    run;
    quit;
      %if &syserr > 4 %then 
    %do;
        %let successYN=N;
        %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
        %goto mExit;
    %end;

    

     %let report_msg_found=0;
         /* Check if there is any processing messages to be displayed */
     proc sql noprint;
         select '1'  
                into :report_msg_found

         from resp_fincen_report_print  
         where response_processing_msg ne ''
         ; 
    quit;

    
     

    proc report data=resp_fincen_report_view nowindows center split='*' ;
         column response_id rr_id  rr_agency_ref_id rr_status_cd 
		 %if %upcase(&response_type_cd) ne XML %then
    	 %do;
               trans_seq_no 
		 %end;
                bsa_id  primary_error_cnt secondary_error_cnt
         %if %str(&report_msg_found) = %str(1) %then
         %do;
              user_field
              response_processing_msg
         %end; 
          ;
         define response_id /width=30 flow;
         define rr_id /flow width=10 "Report ID";
		 %if %upcase(&response_type_cd) ne XML %then
    	 %do;
         define trans_seq_no/"Transaction*Sequence*Number";
		 %end;
         define rr_agency_ref_id /width=20 "DCN in Report";
         define rr_status_cd /width=20 "Report Status" format=$RT_RR_STATUS.;
         
         %if %str(&report_msg_found) = %str(1) %then
         %do;
         define user_field /'User Field in E-File' width=10;
         define response_processing_msg /flow width=50;
          %end;
         title "&form_type_cd Response Report List";      
    run;
    quit;

    %if &syserr > 4 %then 
    %do;
        %let successYN=N;
        %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
        %goto mExit;
    %end;


   
         

    proc report data=resp_fincen_error_print nowindows center split='*'  ;
	%if %upcase(&response_type_cd) ne XML %then
    %do;
         column response_id rr_id trans_seq_no bsa_id  record_type line_no 
                error_code error_type_cd error_desc 
          ;
         define response_id/order flow width=30;
         define rr_id /width=10 flow "Report ID";
         define error_desc /flow width=50;
         define error_type_cd /'Error Type' width=10 format=$resperrcode.;
         define trans_seq_no/'Transaction*Sequence*No' ;
		
    %end;
	%else
    %do;
	column response_id rr_id  /* trans_seq_no  bsa_id */
           error_context error_code error_type_cd error_desc field_reference 
          ;
         define response_id/order flow width=30;
         define rr_id /width=10 flow "Report ID";
         define error_desc /flow width=30;
         define error_type_cd /'Error Type' width=10 format=$resperrcode.;       		 
         define error_context /flow width=30 ;
		 define field_reference /flow width=30 ;
         
	%end;

     title "&form_type_cd Response Error List";     
    run;
    quit;

    %if &syserr > 4 %then 
    %do;
        %let successYN=N;
        %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
        %goto mExit;
    %end;

%end;
           
 %mExit:

  title;
  %put %sysfunc(sasmsg(&smdds,GEN_MAC_END_NOTE,NOQUOTE,&sysmacroname)) ;




%mend;
