/************************************************************************************/
/* Copyright (c) 2014-2018, SAS Institute Inc., Cary, NC, USA, All Rights Reserved  */
/*                                                                                  */
/* NAME:          ecmrr_resp_fincen_sdtm                                            */
/* PRODUCT:       Enterprise Case Management                                        */
/* PURPOSE:       Driver program to process the response files that are             */
/*                from FINCEN SDTM process.                                         */
/* PARAMETERS:                                                                      */
/*                FORM_CONFIG_RK=key of the fincen form (201 for SAR, 202 for NEWCTR*/
/*                                                                                  */
/*                PRINTYN=Y to generate report.                                     */
/*                RENAMEYN=Y to rename the .asc file to .ascp when it is processed  */
/* NOTE:                                                                            */
/************************************************************************************/


%macro ecmrr_resp_fincen_sdtm (      form_config_rk=                
                                   , printYN=Y
                                   , testYN=N
                                   , renameYN=Y 
                                  )
;

   
    %put %sysfunc(sasmsg(&smdds,GEN_MAC_START_NOTE,NOQUOTE,&sysmacroname)) ;

    %local response_extension;
    %if &form_config_rk=201 or  &form_config_rk=202 or &form_config_rk=203 %then
    %do;
         %let response_extension=.acked;
         %let response_type_cd=ASC;
    %end;
    %else 
    %do;
         %let response_extension=.ACKED;
         %let response_type_cd=XML;
    %end;
    %let testYN=%upcase(&testYN);

    %if &testYN ne Y %then %let testYN=N;

    %let printYN=%upcase(&printYN);

    %if &printYN ne Y %then %let printYN=N;

    
    %let renaneYN=%upcase(&renameYN);

    %if &renameYN ne Y %then %let renameYN=N;

    

    %let successYN=Y;

    %let form_path=&ecm_config_path&slash.Source&slash.form_template;

    %local start_dttm end_dttm;

    /* Save the job start time and round it to seconds */
    data _null_;
    start_dttm=int(datetime());
    call symput('start_dttm',put(start_dttm,best.));
    call symput('start_dttm_fmt',put(start_dttm,nldatm20.));
    run;

    %put %sysfunc(sasmsg(&smdds,GEN_TASK_START_TIME_NOTE,NOQUOTE,&sysmacroname,&start_dttm_fmt)) ;

     %if %str(&form_config_rk) = %str() %then 
     %do;
           %put %sysfunc(sasmsg(&smdds,GEN_PARM_FORM_KEY_MISSING_ERROR,NOQUOTE));
           %let successYN=N;
           %goto mExit;
     %end;
 
     %let form_type_cd=;
       proc sql noprint;
       select strip(form_type_cd) into :form_type_cd 
        from ecm_db.form_config 
         where form_config_rk=&form_config_rk
         ;
        
        quit;
       %if %str(&form_type_cd) = %str() %then 
       %do;
           %put %sysfunc(sasmsg(&smdds,GEN_PARM_FORM_KEY_INVALID_ERROR,NOQUOTE));
           %let successYN=N;
           %goto mExit;
       %end; 
       
       %if %symexist(ecm_sdtm_resp_path_%trim(&form_type_cd))=0 %then
       %do;
           %put %sysfunc(sasmsg(&smdds,GEN_MAC_VAR_MISSING_ERROR,NOQUOTE,ecm_sdtm_resp_path_%trim(&form_type_cd)));
           %let successYN=N;
           %goto mExit;
       %end;

 
    
    /* Validate input parameters.                      */

    data _null_;
    length msg $ 4096;
   
    valid=1;
      

   
    if not exist('ECM_RPT.ECMRR_RESP_FINCEN_ERROR_LIST') then
    do;
       valid=0;
       msg=sasmsg("&smdds",'GEN_TABLE_MISSING_ERROR','NOQUOTE','ECM_RPT.ECMRR_RESP_FINCEN_ERROR_LIST');
       put msg;
    end;
    
    if not exist('ECM_RPT.ECMRR_RESP_FINCEN_REPORT_LIST') then
    do;
       valid=0;
       msg=sasmsg("&smdds",'GEN_TABLE_MISSING_ERROR','NOQUOTE','ECM_RPT.ECMRR_RESP_FINCEN_REPORT_LIST');
       put msg;
    end;
    if not exist('ECM_RPT.ECMRR_RESP_FINCEN_SUMMARY') then
    do;
       valid=0;
       msg=sasmsg("&smdds",'GEN_TABLE_MISSING_ERROR','NOQUOTE','ECM_RPT.ECMRR_RESP_FINCEN_SUMMARY');
       put msg;
    end;

 
    if valid=0 then call symput('SUCCESSYN','N');   
    run;
     
    

    %if %upcase(&successYN)=N %then  %goto mExit;
   
    %let response_pgm_prefix=ecmrr_resp;
    %let form_exist=0; 
     proc sql noprint;
      select 1
             , efile_file_prefix
             , "&response_pgm_prefix"||'_'||strip(form_agency_cd)||'_'||strip(form_type_cd) 
             , form_type_cd
             into
               :form_exist 
             , :efile_file_prefix
             , :response_pgm_nm
             , :form_type_cd
         from ecm_db.form_config
         where form_config_rk=&form_config_rk;
        quit;

     %if &form_exist=0 %then
     %do;
         %let ecm_msg=%sysfunc(sasmsg(&smdds,GEN_PARM_FORM_KEY_INVALID_ERROR,NOQUOTE,&FORM_CONFIG_RK)) ;
         %let successYN=N;
         %goto MEXIT;
     %end; 

 
    %if %str(&response_pgm_nm) ne %str()  %then
    %do;
        %let macExistYN=N;
        %ecm_macro_exist(macroName=&response_pgm_nm);
        %if &macExistYN=N %then
        %do;
            %let _ecm_msg=%sysfunc(sasmsg(&smdds,GEN_MAC_MISSING_ERROR,NOQUOTE,&response_pgm_nm)) ;
            %let successYN=N;
            %goto MEXIT;
        %end;
    %end;


    
     %let nfile=0;
     proc sql noprint;
     create table sdtm_list as
     select distinct 
            live.* 
            , udf.udf_value as x_sdtm_efile_nm
     from   ecm_db.efile_live    
                    (sasdatefmt=(coverage_start_dt=&ecm_dtfmt.. coverage_end_dt=&ecm_dtfmt..))
                    as live
          , ecm_db.efile_udf_char_value as udf
          
     where     (live.transmission_id is not null) 
           and live.efile_status_cd='G'
           and live.form_config_rk=&form_config_rk
           and live.efile_rk=udf.efile_rk
           and live.valid_from_dttm=udf.valid_from_dttm
           and udf.udf_nm='X_SDTM_EFILE_NM' 
     ; 
     %let nfile=&sqlobs;
   
     
    %let file_cnt=0; 
    data response_meta_all ;
         if 0 then set ECM_RPT.ECMRR_RESP_FINCEN_SUMMARY 
                                  (keep=response_id response_file_nm response_file_ext response_type_cd);
         retain file_cnt 0;
         set sdtm_list;
         response_file_nm=strip(transmission_id)||"&response_extension"; 
         response_file_full_nm="%trim(&&&ecm_sdtm_resp_path_&form_type_cd)"
                      ||"&slash"||strip(response_file_nm);
         if fileexist(response_file_full_nm);
                      
         file_cnt+1;
         response_id=transmission_id;
         response_file_ext="&response_extension";
         response_type_cd="&response_type_cd";
         call symput('FILE_CNT',put(file_cnt,best.));
       
    run;

    %if &syserr > 4 %then 
    %do;
         %let successYN=N;
         %put %sysfunc(sasmsg(&smdds,GEN_STP_ABORT,NOQUOTE,&sysmacroname)) ;
         %goto mExit;
    %end;

     %if &file_cnt=0 %then %do;
        %put %sysfunc(sasmsg(&smdds,GEN_NO_FILE_NOTE,NOQUOTE)) ;
        %if %upcase(&printYN)=Y %then
        %do;
           
            data _null_;
                 file print notitles;
                 put "No %trim(&form_type_cd) Response Found.";
            run;

        %end;
        %goto mExit;
    %end; 

 
  
    
    %let file_nobs=%ecm_nobs(response_meta_all );

    /* The previous join may generate more records when there are multiple eFile records with the same transmission ID */
    /* In that case, somebody is accidentally applying the wrong transmission ID to the efile. Display a warning.       */

    %if &file_nobs ne &file_cnt %then
    %do;
         %put %sysfunc(sasmsg(&smdds,GEN_RESP_MULTI_EFILE_WARNING,NOQUOTE)) ;
    %end;

    /* Process each response file */
    %do ifile=1 %to &file_nobs;
       
        data response_meta_one;
             set response_meta_all (firstobs=&ifile);
             call symput('response_file_nm',strip(response_file_nm));
             output;
             stop;
        run;

            
       
          
        filename respfref "%trim(&&&ecm_sdtm_resp_path_&form_type_cd)&slash.%trim(&response_file_nm)";
        %&response_pgm_nm( 
                                 indsn_response_meta=response_meta_one
                               , response_type_cd=&response_type_cd
                               , infref=respfref
                               , testYN=&testYN

                               );
         
      
        %if &successYN=N %then %goto mEXIT;

        %if &renameYN=Y %then
        %do;
            filename pfref "%trim(&&&ecm_sdtm_resp_path_&form_type_cd)&slash.%trim(&response_file_nm)p";
            %ecm_movefile(infref=respfref,outfref=pfref,rcvar=RC);
            filename pfref clear;
        %end;

         filename respfref clear;
 
    %end; /* end file loop */
     
    
    /* Report the processing results */
    %ecmrr_resp_fincen_report(  input_type=from_dttm
                               , from_dttm=&start_dttm
                               , form_config_rk=&form_config_rk
                               , printYN=&printYN
                              );
    %if &successYN=N %then %goto mEXIT; 

   

%mexit:

 %if %upcase(&successYN)=N %then 
  %do;
     %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
  %end;

 %if &ecm_debug=N %then 
   %do;
    proc datasets lib=work nolist nowarn;
         delete RESPONSE_META_ALL
                RESPONSE_META_ONE
                _T_RESPONSE_ERROR_CNT
                _1A
                _2A _2B _2C 
                _3A _3B _3C _3D _3E _3F
                _4A _4B  _4C  _4D _4E _4F _4G _4H _4I
                _9Z
                _NEWCTR_ERROR_CODES
                _SAR_ERROR_CODES
                DIRLIST
    ;
    QUIT;
  %end;



  data _null_;
         end_dttm=ceil(datetime());
         duration=end_dttm - &start_dttm;
         call symput('END_DTTM', put(end_dttm,best.));
         call symput('DURATION', put(duration,best.));
         call symput('END_DTTM_FMT', put(end_dttm,nldatm20.));
    run;

   
    %put %sysfunc(sasmsg(&smdds,GEN_TASK_END_TIME_NOTE,NOQUOTE,&sysmacroname,&end_dttm_fmt, &duration)) ;

    %put %sysfunc(sasmsg(&smdds,GEN_MAC_END_NOTE,NOQUOTE,&sysmacroname)) ;


%mend; 
