/***********************************************************************************/
/* Copyright (c) 2014-2018, SAS Institute Inc., Cary, NC, USA, All Rights Reserved */
/*                                                                                 */
/* NAME:          ecmrr_resp_fincen_update                                         */
/* PRODUCT:       Enterprise Case Management                                       */
/* PURPOSE:       summary and save the response data extracted by                  */ 
/*                  ecmrr_resp_fincen_<form>_read.sas to ECM_RPT.                  */
/* PARAMETERS:    INDSN_RESPONSE_META=sas table containing response file info.     */
/*                INDSN_ERROR_CODES=table containing the list of errors.           */
/*                INDSN_1A=table containing record 1A response data.               */
/*                INDSN_3A=table containing record 3A response data.               */
/*                FORM_TYPE_CD=form type code.                                     */ 
/* NOTE:          called by ECMRR_RESP_FINCEN_PROCESS.                             */ 
/***********************************************************************************/


%macro ecmrr_resp_fincen_update(                              
                                  indsn_response_meta=
                                , indsn_error_codes=
                                , indsn_1A=
                                , indsn_3A=
                                , form_type_cd=
                                , testYN=N 
                              );
 

    %local form_config_rk efile_rk response_id 
           count_3A
           p_err_cnt s_err_cnt
    ;
     
    /* extra response file information */

    data _null_;
      set &indsn_response_meta; 
      call symput('efile_rk',strip(put(efile_rk,best.)));
      call symput('form_config_rk',strip(put(form_config_rk,best.)));
      call symput('response_id',strip(response_id));
    run;


    /********************************************************************/
    /* Count primary and secondary errors at efile and report levels.   */
    /********************************************************************/

    /* join with ECM_FORM.<form>_FINCEN_ERROR_CODE to determine if the error is primary or secondary */
    proc sql;
         create table _t_response_error_list as
         select   err.*
                , 1 as COUNT
                , codelist.error_type_cd
         from  &indsn_error_codes as err 
         left join ecm_form.&form_type_cd._fincen_error_code as codelist
         on err.error_code=codelist.error_cd
         order by line_no, error_seq_no
         ;
         %if &sqlrc > 4 %then 
         %do;
             %let successYN=N;
             %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
             %goto mExit;
         %end;
     
    quit;
    /* Summarize the counts by transaction sequence number and error type */
    /* If TRANS_SEQ_NO is empty, it is not a report level error.          */
    /* It should still be counted as e-file level error.                  */

    proc  summary data=_t_response_error_list missing;
         class trans_seq_no error_type_cd;
         var count; 
         output out=_t_response_error_cnt (where=(_type_ in (1, 3)))  sum=count;
    run;

    %if &syserr > 4 %then 
    %do;
        %let successYN=N;
        %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
        %goto mExit;
    %end;



    /* Transpose the prmary and secondary count records to columns at report level */ 
     
     
    %let p_err_cnt=0;
    %let s_err_cnt=0;
    data _t_response_error_transposed;
         set _t_response_error_cnt;
         by trans_seq_no;
         where trans_seq_no > 0 and _type_=3; /* include only report level error */
          length PRIMARY_ERROR_CNT 
                 SECONDARY_ERROR_CNT 
                 8;
          retain primary_error_cnt 
                 secondary_error_cnt 
          ;
         rr_rk=.; /* initialize RR_RK for future update */
         if first.trans_seq_no then 
         do;
            primary_error_cnt=0;
            secondary_error_cnt=0;
         end;

       
         if error_type_cd='P' then primary_error_cnt=count;
         if error_type_cd='S' then secondary_error_cnt=count;
           
         /* Only at report level */
         if last.trans_seq_no then output;
    run;

    %if &syserr > 4 %then 
    %do;
        %let successYN=N;
        %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
        %goto mExit;
    %end;
     
    /********************************************************************/
    /* Create response summary table.                                   */
    /********************************************************************/

    /* count 3A records */
    %let count_3A=%ecm_nobs(&indsn_3A);
    /* count 1A records */
    %let count_1A=%ecm_nobs(&indsn_1A);

    /* Retrieve the primary and secondary error count at the efile level */
    data _null_;
        set _t_response_error_cnt;
        where _type_=1;
        if error_type_cd='P' then call symput('P_ERR_CNT',put(count,best.)); 
        if error_type_cd='S' then call symput('S_ERR_CNT',put(count,best.)); 
    run;

    /* create the final ECMRR_RESP_FINCEN_SUMMARY with the validation messages and error counts */ 

    data ECMRR_RESP_FINCEN_SUMMARY;
         if 0 then set ecm_rpt.ecmrr_resp_fincen_summary;
         merge &indsn_response_meta (keep=response_id response_type_cd response_file_nm response_file_ext 
                                          efile_rk form_config_rk coverage_start_dt coverage_end_dt output_rr_cnt)
                &indsn_1A (keep=coverage_begin_dt coverage_end_dt
                %if ( (%upcase(&form_type_cd) = CTRX) or (%upcase(&form_type_cd) = SARX) ) %then
                %do;       
                        submission_status_cd
                %end;
                    rename=(coverage_begin_dt=coverage_start_dt_str 
                           coverage_end_dt=coverage_end_dt_str
                            )
                   ) 
                ;
        
         record_3A_cnt=&count_3A;
		 record_1A_cnt=&count_1A;
         primary_error_cnt=&p_err_cnt;
         secondary_error_cnt=&s_err_cnt;
         processed_dttm=datetime();
		 /* Add some specific XML processing tests. */
		 if "%upcase(&form_type_cd)" in ("CTRX","SARX") then 
		     do;
             response_processing_msg='';
             match_flg=1;	
             /* if the 3A and 1A datasets are 0 records, chances are the XML file is not valid format, since
                3A has the BSAID tag, and 1A has Submission status, coverage dates and record type. */
             if record_3A_cnt eq 0 and record_1A_cnt eq 0 then 
		         do;
                 match_flg=0;
                 response_processing_msg='Invalid formatted XML file, missing BSAID tag.'; 
				 return;
                 end;
             /* check for matching efile */
             if efile_rk = . then
                 do;
                 match_flg=0;
                 response_processing_msg='No matching e-file.'; 
				 return;
                 end;
		     end;
		 else
             link VALIDATE_EFILE_META; /* Confirm that response data matches the efile data */
         return;
          
         drop match_flg coverage_start_dt coverage_end_dt output_rr_cnt;

    VALIDATE_EFILE_META:
        response_processing_msg='';
        match_flg=1;	
		
        /* check for matching efile */
        if efile_rk = . then
        do;
               match_flg=0;
               response_processing_msg='No matching e-file.'; 
               return;
        end;
        /* check for matching 3A count */
        if     record_3A_cnt ne output_rr_cnt then
        do;
               match_flg=0;
               response_processing_msg=strip(response_processing_msg)||' Report count not matched.'; 
        end;
        
		   
        if    coverage_start_dt_str ne put(coverage_start_dt,mmddyyn8.) 
           or coverage_end_dt_str ne put(coverage_end_dt,mmddyyn8.) then
        do;
            
             response_processing_msg=strip(response_processing_msg)||' Coverage Period not matched.'; 
        end;                   
    return;
	
    run;

    %if &syserr > 4 %then 
    %do;
        %let successYN=N;
        %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
        %goto mExit;
    %end;

     
    /********************************************************************/
    /* Create response report list table.                               */
    /********************************************************************/

    /* Get the definition of user field of 3A record in form configuration table                      */
    /* ECM provided definition is RR_ID.                                                              */ 
    /* However, the site may change it to RR_RK if it is anticipated that the ID can be longer than   */
    /* 10 characters which is the size limit of user field.                                           */ 
    /* The user field is used to confirm that the correct report is matched.                          */ 
    /* Note: CTRX and SARX will alway use RR_RK as seqNum .                                           */
%if ( %str(%upcase(&form_type_cd))=%str(CTRX) or 
      %str(%upcase(&form_type_cd))=%str(SARX) ) %then %let user_field_nm=RR_RK;
%else
%do; 
    proc sql noprint;
         select     strip(upcase(source_field_nm))             
                 into
                  : user_field_nm
                
         from ecm_db.form_efile_field_config as fld, 
              ecm_db.form_efile_record_config as rec 
         where     fld.form_config_rk=&form_config_rk
               and rec.form_config_rk=&form_config_rk
               and rec.form_efile_record_id='3A' 
               and fld.form_efile_record_seq_no=rec.form_efile_record_seq_no
               and upcase(fld.target_field_nm)='USER FIELD'
          ;
         %if &sqlrc > 4 %then 
         %do;
             %let successYN=N;
             %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
             %goto mExit;
         %end;
    quit;
%end;

    /* create a format for RR_RK look up based on the user field definition */ 

    %if %str(&user_field_nm)=%str(RR_ID) %then 
    %do;
        data id_rk_fmt;
             set ecm_db.rr_live (keep=rr_id rr_rk efile_rk) end=eof;
             where efile_rk=&efile_rk;
             retain fmtname 'id_rk_fmt'
                    type    'C'
             ;
             length start end label $40; 
             start=rr_id;
             end=start;
             label=left(put(rr_rk,10.));
             output;
             if eof then 
             do;
                 start='**OTHER**';
                 hlo='O';
                 label='INVALID';
                 output;
            end;  
        run;
        %if &syserr > 4 %then 
        %do;
            %let successYN=N;
            %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
            %goto mExit;
        %end;

        
        proc format cntlin=id_rk_fmt;
        run;
        %if &syserr > 4 %then 
        %do;
            %let successYN=N;
            %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
            %goto mExit;
        %end;

        

    %end;

    %if %str(&user_field_nm)=%str(RR_RK) %then 
    %do;
        data rk_id_fmt;
             set ecm_db.rr_live (keep=rr_id rr_rk efile_rk) end=eof;
             where efile_rk=&efile_rk;
             retain fmtname 'rk_id_fmt'
                    type    'N'
             ;
             length start end label $40; 
             start=put(rr_rk,best.);
             end=start;
             label=rr_id;
             output;
             if eof then 
             do;
                 start='**OTHER**';
                 hlo='O';
                 label='INVALID';
                 output;
             end; 
        run;
        %if &syserr > 4 %then 
        %do;
            %let successYN=N;
            %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
            %goto mExit;
        %end;

        

        proc format cntlin=rk_id_fmt;
        run;

        %if &syserr > 4 %then 
        %do;
            %let successYN=N;
            %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
            %goto mExit;
        %end;

        
    %end;

 
 
    /* Join the 3A records with the RR_LIVE by transaction sequence number */
    /* to get RR_RK.                                                       */
    /* This join works only if the transaction sequence number is saved    */
    /* to the ECM_DB.RR_LIVE.                                              */
    /* PROC SOAP is working with the correct credential when the efile     */
    /* was generated.                                                      */
    
    
    proc sql;
         create table _t_response_report_list as
         select   db_rr.rr_rk as RR_RK
                , db_rr.rr_id 
                , resp_rr.*   
         from  &indsn_3A as resp_rr
         left join ecm_db.rr_live as db_rr
		 /* With XML submitted response files, cannot rely on the returned order of efiles.  */
		 /* Will join based on report key instead of efile seq number = response seq number. */
		 %if ( (%upcase(&form_type_cd) = CTRX) or (%upcase(&form_type_cd) = SARX) ) %then
            %do;
			on input(resp_rr.user_field,11.) = db_rr.rr_rk
			%end;
		 %else %do;
            on resp_rr.trans_seq_no=db_rr.efile_report_seq_no
		 %end;
            and db_rr.efile_rk=&efile_rk
         order by trans_seq_no
         ;
         %if &sqlrc > 4 %then 
         %do;
             %let successYN=N;
             %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
             %goto mExit;
         %end;
 
   quit;

    /* Confirm that the correct report pulled by checking the 3A record user field */
    data _t_response_report_list_2;
         if 0 then set ECM_RPT.ECMRR_RESP_FINCEN_REPORT_LIST; 
         set _t_response_report_list;
         efile_rk=symgetn('EFILE_RK');
         response_id=symgetc('RESPONSE_ID');
         matched_flg=0;
         primary_error_cnt=0;
         secondary_error_cnt=0;
         %if %str(&user_field_nm)=%str(RR_ID) %then 
         %do;
            if rr_rk=. then 
            do;
              if  put(user_field,$id_rk_fmt.) ne 'INVALID' then
              do;
                  rr_rk=input(put(user_field,$id_rk_fmt.),best.);
                  matched_flg=1;
              end;
            end;
            else if user_field=substr(rr_id,1,10) then matched_flg=1;
         %end;
         %if %str(&user_field_nm)=%str(RR_RK) %then 
         %do;
            if rr_rk=. then 
            do;
               rr_rk=input(user_field,best.);
               if put(rr_rk,rk_id_fmt.) ne 'INVALID' then 
               do; 
                   rr_id=put(rr_rk,rk_id_fmt.);
                   matched_flg=1;
               end;
            end;
            else if input(user_field,best.)=rr_rk then matched_flg=1;
         %end;
         if matched_flg=0 then response_processing_msg="3A User field does not match with %trim(&user_field_nm)";
         keep response_id efile_rk trans_seq_no rr_rk bsa_id 
              user_field response_processing_msg;

    run;

    %if &syserr > 4 %then 
    %do;
        %let successYN=N;
        %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
        %goto mExit;
    %end;

    
     
    /* Create the final ECMRR_RESP_FINCEN_REPORT_LIST with the error counts */
    proc sql;
        create table ECMRR_RESP_FINCEN_REPORT_LIST  as 
        select  rpt.*
            , coalesce(err.primary_error_cnt,0) as primary_error_cnt
            , coalesce(err.secondary_error_cnt,0) as secondary_error_cnt
        from _t_response_report_list_2 as rpt
        left join _t_response_error_transposed as err
        on  rpt.trans_seq_no=err.trans_seq_no
        ; 
   
     %if &sqlrc > 4 %then 
     %do;
        %let successYN=N;
        %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
        %goto mExit;
    %end;
 


    /* Create the final ECMRR_RESP_FINCEN_ERROR_LIST with RR info */  
    
        create table ECMRR_RESP_FINCEN_ERROR_LIST  as 
        select 
                 "&response_id" as RESPONSE_ID length=256
                , &efile_rk as EFILE_RK
				/*
                , err.line_no                       
                , err.error_seq_no                  
                , err.trans_seq_no                   
                , err.record_type                    
                , err.bsa_id                        
                , err.error_code                     
                , err.error_type_cd                
                , err.user_field
                */
                , err.* 
            , coalesce(rpt.rr_rk) as rr_rk
        from _t_response_error_list as err
        left join ECMRR_RESP_FINCEN_REPORT_LIST  as rpt
        on  rpt.trans_seq_no=err.trans_seq_no
        order by line_no, error_seq_no
        ; 
    %if &sqlrc > 4 %then 
    %do;
        %let successYN=N;
        %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
        %goto mExit;
    %end;
 

    

    /* Remove any previously loaded response data for the same response file */
    /* Then append the new response data.                                    */          

     
         delete 
         from ECM_RPT.ECMRR_RESP_FINCEN_SUMMARY 
         where response_id="&response_id";
         delete 
         from ECM_RPT.ECMRR_RESP_FINCEN_REPORT_LIST 
         where response_id="&response_id";
         delete 
         from ECM_RPT.ECMRR_RESP_FINCEN_ERROR_LIST 
         where response_id="&response_id"
         ;
    %if &sqlrc > 4 %then 
    %do;
        %let successYN=N;
        %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
        %goto mExit;
    %end;


    quit;

    


    proc append base=ECM_RPT.ECMRR_RESP_FINCEN_SUMMARY  
                data=ECMRR_RESP_FINCEN_SUMMARY force;
    run;
    %if &syserr > 4 %then 
    %do;
        %let successYN=N;
        %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
        %goto mExit;
    %end;

    proc append base=ECM_RPT.ECMRR_RESP_FINCEN_REPORT_LIST  
                data=ECMRR_RESP_FINCEN_REPORT_LIST force;
    run;

    %if &syserr > 4 %then 
    %do;
        %let successYN=N;
        %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
        %goto mExit;
    %end;

    
    proc append base=ECM_RPT.ECMRR_RESP_FINCEN_ERROR_LIST  
                data=ECMRR_RESP_FINCEN_ERROR_LIST force;
    run;

    %if &syserr > 4 %then 
    %do;
        %let successYN=N;
        %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
        %goto mExit;
    %end;
   
    %ecmrr_resp_fincen_updt_ecm_db(  resp_summary_dsn=ECMRR_RESP_FINCEN_SUMMARY
                                    ,resp_report_dsn=ECMRR_RESP_FINCEN_REPORT_LIST
                                    ,testYN=&testYN
                                    ,resp_summary_where=%nrbquote(response_processing_msg='')
                                    ,resp_report_where=%nrbquote(response_processing_msg='')
                                    
                                  );
 
    %if &successYN=N %then %goto mEXIT; 


%mExit:

%if &ecm_debug=N %then 
%do;
    proc datasets lib=work nolist nowarn;
         delete _T_RESPONSE_ERROR_LIST
                _T_RESPONSE_REPORT_LIST
                _T_RESPONSE_REPORT_LIST_2
                _T_RESPONSE_ERROR_TRANSPOSED
                ECMRR_RESP_FINCEN_SUMMARY
                ECMRR_RESP_FINCEN_REPORT_LIST
                ECMRR_RESP_FINCEN_ERROR_LIST
                ID_RK_FMT
                
    ;
    QUIT;
%end;

%mend;
