/*----------------------------------------------------------------------------------------
 *    NAME: ecmrr_validate_xml.sas
 *   TITLE: ECM XML Report Validator against FinCEN provided XSD schema files.
 * PRODUCT: SAS Currency Transaction Reporting
 *  SYSTEM: UNIX/Windows
 *---------------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------------
 * Copyright (c) 2018 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *---------------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------------------
 *    This is a sample macro to validate a generated XML report against the FinCEN
 *    provided XSD Schema file to minimize the number of e-file rejections due to
 *    invalid XML files.
 *
 *    Requirement:
 *        A version of Java 7 or 8 JRE is required to execute the supplied JAR file and to
 *        resolve the path to FinCEN provided repositories. SAS supplied Java 1.7 will be the default.
 *
 *    Inputs:
 *        xml_file          = full path location of XML file to validate
 *        xsd_file          = full path location of XSD to validate against
 *        validation_output = name of validation stdout/stderr file created in WORK
 *        2 parameters with defaults defined in macro parameter definitions
 *        --------------------------------------------------------
 *        java_loc          = fully qualified path for JAVA 7/8 executable
 *        jar_loc           = fully qualified path for the XML validation jar file
 *
 *    Output:
 *        SYSRC from X command will contain 0 if validation is clean; 1 if error found
 *           and error details will be found in the validation_details dataset. 
 *        And the XML source file will be saved in xmlsource dataset that can be displayed.
 *---------------------------------------------------------------------------------------*/

%macro ecmrr_validate_xml(xml_file=
                        , xsd_file=&ecm_config_path.&slash.Source&slash.form_template&slash.schema&slash.FINCEN&slash.ctrx_schema.xsd
                        , java_loc=
                        , jar_loc=&ecm_config_path.&slash.Source&slash.form_template&slash.xmlvalidator.jar
                        , out_file=_webout);

   %put %sysfunc(sasmsg(&smdds,GEN_MAC_START_NOTE,NOQUOTE,&sysmacroname));


  /*---------------------------------------------------------------------------------------
   *   For now, will set some site specific defaults to find Java and location of Jar file
   *--------------------------------------------------------------------------------------*/
   %let successYN=Y;
   %let validationRC=0;
   %let valoptionsRC=0;
   %let validation_output=val_out.txt;
   %let win_cmd=val_cmd.bat;

   /* if java_loc is NOT passed (blank), build the path from SAS default environment variable */
   data _null_;
       format homeval setpath $256.;
       homeval=trim(sysget('JAVA_HOME'));
       passed_val=trim(symget('JAVA_LOC'));
       if passed_val = '' then
          setpath = cat(trim(homeval),"&slash.","bin","&slash.","java");
       else
          setpath = trim(passed_val);
       call symput('JAVA_LOC',trim(setpath));
   run;

   /* Check for the XML and XSD file existence before proceeding.  */
   /* If not found, then going ahead and stop before attempting.   */
   %if %sysfunc(fileexist("&xml_file.")) eq 0 %then
      %let valoptionsRC=1;
   %if %sysfunc(fileexist("&xsd_file.")) eq 0 %then
      %let valoptionsRC=2;
   %if %sysfunc(fileexist("&java_loc.")) eq 0 %then
      %let valoptionsRC=3;
   %if %sysfunc(fileexist("&jar_loc.")) eq 0 %then
      %let valoptionsRC=4;

   /* Worst case scenario on not finding options, failed to pass back up the error flag */
   %if &valoptionsRC > 0 %then
      %let successYN=N;

   /* Create the validation output filename value in WORK location. */
   data _null_;
      format val_file workdir $256.;
      workdir=trim(getoption('WORK'));
      val_file=trim(workdir) || "&slash." || trim("&validation_output.");
      win_file=trim(workdir) || "&slash." || trim("&win_cmd.");
      call symput('VALFILE',trim(val_file));
      call symput('BATFILE',trim(win_file));
   run;


   /* Quote the output file parameter if the value is NOT _webout (default) */
   %if %upcase("&out_file.")="_WEBOUT" %then %do;
       %let out_fpath = &out_file.;
       %let hdron=<h3>;
       %let hdroff=</h3>;
       %let brtag="<br>";
       %let sptag="<p>";
       %end;
   %else %do;
       %let out_fpath="&out_file.";
       %let hdron=;
       %let hdroff=;
       %let brtag=;
       %let sptag=;
       %end;

   %if &valoptionsRC eq 0 %then %do;
       /* Using Filename Pipe.  X command will not conditionally run.  %SYSEXE will only run Java that is default java */
       %if &sysscp eq WIN %then %do;
           /* Build the command in datastep to double quote file pathnames in case this is Windows so the quoting in */
           /* pipe command will be correct. Double each option that could be a pathname.                             */
           data _null_;
              format full_command $512.;
              file "&batfile.";
              full_command=cat("",'"',"&java_loc.",'"'
                                 ,' -jar '
                                 ,'"',"&jar_loc.",'"'
                                 ,' "',"&xml_file.",'"'
                                 ,' "',"&xsd_file.",'"'
                                 ,' >','"',"&valfile.",'"',' 2>&1'
                                 ,"");
              call symput('pipecmd',trim(full_command));
              put full_command;
              run;
           /* single quote the command and double quote the path */
           data _null_;
               format full_cmd_path $512.;
               full_cmd_path=trim(cat("'",'"',"&batfile.",'"',"'"));
               call symput('BATCH_CMD',full_cmd_path);
               run;
           filename runval pipe &batch_cmd.;
           %end;
        %else %do;
           /* Unix will not need the double quoting as in windows. */
           data _null_;
              format full_command $512.;
              full_command=cat("&java_loc."
                                 ,' -jar '
                                 ,"&jar_loc."
                                 ,' ',"&xml_file."
                                 ,' ',"&xsd_file."
                                 ,' >',"&valfile.",' 2>&1'
                                 );
              call symput('pipecmd',trim(full_command));
              put full_command;
              run;
           filename runval pipe "&pipecmd.";
           %end;
       data _null_;
           infile runval lrecl=512 recfm=v missover;
           input @1 line $512.;
           /*put line=;*/
       run;
       /* Check the output file for "xxxx is valid." for successful. */
       %if %sysfunc(fileexist("&valfile.")) eq 0 %then %do;
           %let validationRC=2;
           %GOTO MACRO_END;
           %end;
       /* Read the output file and look for is valid to set RC to 0. */
       %else %do;
           filename look "&valfile.";
           data validation_details (drop=start_of_line);
              format aline $512.;
              infile look pad missover lrecl=512 recfm=v end=done;
              file &out_fpath.;
              input @1 aline $512.;
              if _n_=1 and index(aline,'is valid.') > 0 then do;
                 call symput('validationRC',0);
                 stop;
                 end;
              else do;
                 call symput('validationRC',1);
                 if _n_=1 then do;
                    put "&hdron.Following Error(s) found with XML Validation:&hdroff.";
                    end;
				 start_of_line=index(aline,'lineNumber:');
                 if start_of_line > 0 then do;
                    aline=substr(aline,start_of_line);
                    end;
                 if done then
                    put &sptag.;
                 if index(aline,'at com.sas.xmlvalidator.') > 0 then delete;
                 put aline &brtag.;
                 end;

           run;
		   /* if parsing errors are found, create a dataset of lines in error to flag */
		   data error_lines (keep=error_line_number);
		      format error_line_number 8. startpos endpos 8.;
			    set validation_details;
			    if index(aline,'org.xml.sax.SAXParseException') > 0 then do;
			       startpos=index(aline,'lineNumber:') + 12; 
				     endpos=index(aline,'; columnNumber:');
				     error_line_number=input(substr(aline,startpos,endpos-startpos),8.);
				     output;
			       end;
		    run;
           filename look clear;
           %end;
       %end;

   %if ((&validationRC eq 0) && (&valoptionsRC eq 0)) %then %do;
       %put Note: Validation of &xml_file. was successful.;
       data _null_;
          file &out_fpath.;
          put "&hdron.Validation was successful.&hdroff.";
          put &sptag.;
       run;
       %end;
   %else %if ((&validationRC eq 1) && (&valoptionsRC eq 0)) %then %do;
       /* Process a copy of the original XML Source file to return to user. */
	     filename inxml "&xml_file.";
	     data xmlsource;
	        infile inxml pad missover recfm=v lrecl=2048;
		      format row_number 8. xml_src_line $2048. error_line_flag $1.;
		      input @1 xml_src_line $2048.;
		      row_number = _n_;
	     run;
	     /* Set the error_line_flag on xml source dataset where XML validation errors were found. */
	     proc sort data=error_lines nodupkey;
	        by error_line_number;
	     run;   
       proc sql noprint;
          update xmlsource 
             set error_line_flag = 'T'
                where row_number in (select error_line_number from error_lines);
       quit;				
	   
	     filename inxml clear;
       %put NOTE: Validation of &xml_file. failed. Error details in WORK.Validation_Details dataset.;
       %end;

%MACRO_END:

    %if &valoptionsRC=1 %then %do;
        %put ERROR: Unable to locate and open XML file passed with xml_file option.;
        %end;
    %if &valoptionsRC=2 %then %do;
        %put ERROR: Unable to locate and open XSD file passed with xsd_file option.;
        %end;
    %if &valoptionsRC=3 %then %do;
        %put ERROR: Unable to locate Java executable passed with java_loc option.;
        %end;
    %if &valoptionsRC=4 %then %do;
        %put ERROR: Unable to locate xmlvalidator.jar file passed with jar_loc option.;
        %end;
    %if &validationRC=2 %then %do;
        %put ERROR: Problem executing the validation jar file. No output found.;
        %end;

    %put %sysfunc(sasmsg(&smdds,GEN_MAC_END_NOTE,NOQUOTE,&sysmacroname));
	

%mend ecmrr_validate_xml;
